import decimal
import decimal
import math

from apexpro.constants import URL_SUFFIX, ORDER_SIDE_BUY
from apexpro.helpers.request_helpers import random_client_id, iso_to_epoch_seconds, epoch_seconds_to_iso
from apexpro.http_private import HttpPrivate
from apexpro.starkex.conditional_transfer import SignableConditionalTransfer
from apexpro.starkex.constants import ONE_HOUR_IN_SECONDS, ORDER_SIGNATURE_EXPIRATION_BUFFER_HOURS
from apexpro.starkex.helpers import get_transfer_erc20_fact, nonce_from_client_id
from apexpro.starkex.order import SignableOrder, DECIMAL_CONTEXT_ROUND_UP, DECIMAL_CONTEXT_ROUND_DOWN
from apexpro.starkex.transfer import SignableTransfer
from apexpro.starkex.withdrawal import SignableWithdrawal


class HttpPrivateStark(HttpPrivate):
    def create_order(self,
                     symbol,
                     side,
                     type,
                     size,
                     limitFeeRate=None,
                     limitFee=None,
                     price=None,
                     accountId=None,
                     timeInForce="GOOD_TIL_CANCEL",
                     reduceOnly=False,
                     triggerPrice=None,
                     triggerPriceType="INDEX",
                     trailingPercent=None,
                     clientId=None,
                     expiration=None,
                     expirationEpochSeconds=None,
                     isPositionTpsl = False,
                     signature=None,
                     isOpenTpslOrder=False,
                     isSetOpenSl=False,
                     isSetOpenTp=False,
                     slClientId=None,
                     slPrice=None,
                     slSide=None,
                     slSize=None,
                     slTriggerPrice=None,
                     tpClientId=None,
                     tpPrice=None,
                     tpSide=None,
                     tpSize=None,
                     tpTriggerPrice=None,
                     sourceFlag=None,
                     brokerId=None,):
        """"
        POST  create_order.
        client.create_order(symbol="BTC-USDC", side="SELL",
                                           type="LIMIT", size="0.01",
                                           price="20000", limitFeeRate="1",
                                            accountId="325881046451093849",reduceOnly=False,
                                           expiration=now_iso + SEVEN_DAYS_S + 60*1000, timeInForce="GOOD_TIL_CANCEL")

       :returns: Request results as dictionary.
       """
        price = str(price)
        size = str(size)
        clientId = clientId or random_client_id()
        if not self.stark_private_key:
            raise Exception(
                'No signature provided and client was not ' +
                'initialized with stark_private_key'
            )

        if bool(expiration) == bool(expirationEpochSeconds):
            raise ValueError(
                'Exactly one of expiration and expiration_epoch_seconds must '
                'be specified',
            )
        expiration = expiration or epoch_seconds_to_iso(
            expirationEpochSeconds,
        )
        expirationEpochSeconds = (
                expirationEpochSeconds or iso_to_epoch_seconds(expiration)
        )
        limitFeeRate = limitFeeRate or limitFee
        accountId = accountId or self.account.get('positionId')
        if not accountId:
            raise Exception(
                'No accountId provided' +
                'please call get_account()'
            )

        if not self.config:
            raise Exception(
                'No config provided' +
                'please call configs()'
            )
        symbolData = {}
        currency = {}
        for k, v in enumerate(self.config.get('perpetualContract')):
            if v.get('symbol') == symbol:
                symbolData = v
        for k, v2 in enumerate(self.config.get('currency')):
            if v2.get('id') == symbolData.get('settleCurrencyId'):
                currency = v2

        if symbolData is not None :
            number = decimal.Decimal(price) / decimal.Decimal(symbolData.get('tickSize'))
            if number > int(number):
                raise Exception(
                    'the price must Multiple of tickSize'
                )

        order_to_sign = SignableOrder(
            position_id=accountId,
            client_id=clientId,
            market=symbol,
            side=side,
            human_size=size,
            human_price=price,
            limit_fee=limitFeeRate,
            expiration_epoch_seconds=expirationEpochSeconds,
            synthetic_resolution=symbolData.get('starkExResolution'),
            synthetic_id=symbolData.get('starkExSyntheticAssetId'),
            collateral_id=currency.get('starkExAssetId'),
        )
        signature = order_to_sign.sign(self.stark_private_key)

        if side == ORDER_SIDE_BUY:
            human_cost = DECIMAL_CONTEXT_ROUND_UP.multiply(
                decimal.Decimal(size),
                decimal.Decimal(price)
            )
            fee = DECIMAL_CONTEXT_ROUND_UP.multiply(human_cost, decimal.Decimal(limitFeeRate))
        else:
            human_cost = DECIMAL_CONTEXT_ROUND_DOWN.multiply(
                decimal.Decimal(size),
                decimal.Decimal(price)
            )
            fee = DECIMAL_CONTEXT_ROUND_DOWN.multiply(human_cost, decimal.Decimal(limitFeeRate))

        limit_fee_rounded = DECIMAL_CONTEXT_ROUND_UP.quantize(
            decimal.Decimal(fee),
            decimal.Decimal(currency.get('stepSize')), )
        expirationEpoch = math.ceil(
            float(expirationEpochSeconds) / ONE_HOUR_IN_SECONDS,
        ) + ORDER_SIGNATURE_EXPIRATION_BUFFER_HOURS

        sl_limit_fee_rounded = None
        slSignature = None
        slTriggerPriceType = None
        slExpiration = None
        tp_limit_fee_rounded = None
        tpSignature = None
        tpTriggerPriceType = None
        tpExpiration = None

        if isOpenTpslOrder == True:
            if isSetOpenSl == True:
                slTriggerPriceType = triggerPriceType
                slExpiration = expirationEpoch * 3600 * 1000
                slClientId = slClientId or random_client_id()
                slOrder_to_sign = SignableOrder(
                    position_id=accountId,
                    client_id=slClientId,
                    market=symbol,
                    side=slSide,
                    human_size=slSize,
                    human_price=slPrice,
                    limit_fee=limitFeeRate,
                    expiration_epoch_seconds=expirationEpochSeconds,
                    synthetic_resolution=symbolData.get('starkExResolution'),
                    synthetic_id=symbolData.get('starkExSyntheticAssetId'),
                    collateral_id=currency.get('starkExAssetId'),
                )
                slSignature = slOrder_to_sign.sign(self.stark_private_key)

                if slSide == ORDER_SIDE_BUY:
                    slHuman_cost = DECIMAL_CONTEXT_ROUND_UP.multiply(
                    decimal.Decimal(slSize),
                    decimal.Decimal(slPrice)
                    )
                    slFee = DECIMAL_CONTEXT_ROUND_UP.multiply(slHuman_cost, decimal.Decimal(limitFeeRate))
                else:
                    slHuman_cost = DECIMAL_CONTEXT_ROUND_DOWN.multiply(
                        decimal.Decimal(slSize),
                        decimal.Decimal(slPrice)
                    )
                    slFee = DECIMAL_CONTEXT_ROUND_DOWN.multiply(slHuman_cost, decimal.Decimal(limitFeeRate))

                sl_limit_fee_rounded = DECIMAL_CONTEXT_ROUND_UP.quantize(
                    decimal.Decimal(slFee),
                    decimal.Decimal(currency.get('stepSize')), )

            if isSetOpenTp == True:
                tpTriggerPriceType = triggerPriceType
                tpExpiration = expirationEpoch * 3600 * 1000
                tpClientId = tpClientId or random_client_id()
                tpOrder_to_sign = SignableOrder(
                    position_id=accountId,
                    client_id=tpClientId,
                    market=symbol,
                    side=tpSide,
                    human_size=tpSize,
                    human_price=tpPrice,
                    limit_fee=limitFeeRate,
                    expiration_epoch_seconds=expirationEpochSeconds,
                    synthetic_resolution=symbolData.get('starkExResolution'),
                    synthetic_id=symbolData.get('starkExSyntheticAssetId'),
                    collateral_id=currency.get('starkExAssetId'),
                )
                tpSignature = tpOrder_to_sign.sign(self.stark_private_key)

                if tpSide == ORDER_SIDE_BUY:
                    tpHuman_cost = DECIMAL_CONTEXT_ROUND_UP.multiply(
                        decimal.Decimal(tpSize),
                        decimal.Decimal(tpPrice)
                    )
                    tpFee = DECIMAL_CONTEXT_ROUND_UP.multiply(tpHuman_cost, decimal.Decimal(limitFeeRate))
                else:
                    tpHuman_cost = DECIMAL_CONTEXT_ROUND_DOWN.multiply(
                        decimal.Decimal(tpSize),
                        decimal.Decimal(tpPrice)
                    )
                    tpFee = DECIMAL_CONTEXT_ROUND_DOWN.multiply(tpHuman_cost, decimal.Decimal(limitFeeRate))

                tp_limit_fee_rounded = DECIMAL_CONTEXT_ROUND_UP.quantize(
                    decimal.Decimal(tpFee),
                    decimal.Decimal(currency.get('stepSize')), )

        order = {
            'symbol': symbol,
            'side': side,
            'type': type,
            'timeInForce': timeInForce,
            'size': size,
            'price': price,
            'limitFee': str(limit_fee_rounded),
            'expiration': expirationEpoch * 3600 * 1000,
            'triggerPrice': triggerPrice,
            'triggerPriceType': triggerPriceType,
            'trailingPercent': trailingPercent,
            'clientId': clientId,
            'signature': signature,
            'reduceOnly': reduceOnly,
            'isPositionTpsl': isPositionTpsl,
            'isOpenTpslOrder': isOpenTpslOrder,
            'isSetOpenSl': isSetOpenSl,
            'isSetOpenTp': isSetOpenTp,
            'slClientOrderId': slClientId,
            'slPrice': slPrice,
            'slSide': slSide,
            'slSize': slSize,
            'slTriggerPrice': slTriggerPrice,
            'slTriggerPriceType': slTriggerPriceType,
            'slExpiration': slExpiration,
            'slLimitFee': str(sl_limit_fee_rounded),
            'slSignature': slSignature,
            'tpClientOrderId': tpClientId,
            'tpPrice': tpPrice,
            'tpSide': tpSide,
            'tpSize': tpSize,
            'tpTriggerPrice': tpTriggerPrice,
            'tpTriggerPriceType': tpTriggerPriceType,
            'tpExpiration': tpExpiration,
            'tpLimitFee': str(tp_limit_fee_rounded),
            'tpSignature': tpSignature,
            'sourceFlag': sourceFlag,
            'brokerId':brokerId,
        }

        path = URL_SUFFIX + "/v1/create-order"
        return self._post(
            endpoint=path,
            data=order
        )

    def create_order_v2(self,
                     symbol,
                     side,
                     type,
                     size,
                     limitFeeRate=None,
                     limitFee=None,
                     price=None,
                     accountId=None,
                     timeInForce="GOOD_TIL_CANCEL",
                     reduceOnly=False,
                     triggerPrice=None,
                     triggerPriceType=None,
                     trailingPercent=None,
                     clientId=None,
                     expiration=None,
                     expirationEpochSeconds=None,
                     isPositionTpsl = False,
                     signature=None,
                     isOpenTpslOrder=False,
                     isSetOpenSl=False,
                     isSetOpenTp=False,
                     slClientId=None,
                     slPrice=None,
                     slSide=None,
                     slSize=None,
                     slTriggerPrice=None,
                     tpClientId=None,
                     tpPrice=None,
                     tpSide=None,
                     tpSize=None,
                     tpTriggerPrice=None,
                     sourceFlag=None,
                     brokerId=None,):
        """"
        POST  create_order.
        client.create_order(symbol="BTC-USDC", side="SELL",
                                           type="LIMIT", size="0.01",
                                           price="20000", limitFeeRate="1",
                                            accountId="325881046451093849",reduceOnly=False,
                                           expiration=now_iso + SEVEN_DAYS_S + 60*1000, timeInForce="GOOD_TIL_CANCEL")

       :returns: Request results as dictionary.
       """
        price = str(price)
        size = str(size)
        clientId = clientId or random_client_id()
        if not self.stark_private_key:
            raise Exception(
                'No signature provided and client was not ' +
                'initialized with stark_private_key'
            )

        if bool(expiration) == bool(expirationEpochSeconds):
            raise ValueError(
                'Exactly one of expiration and expiration_epoch_seconds must '
                'be specified',
            )
        expiration = expiration or epoch_seconds_to_iso(
            expirationEpochSeconds,
        )
        expirationEpochSeconds = (
                expirationEpochSeconds or iso_to_epoch_seconds(expiration)
        )
        limitFeeRate = limitFeeRate or limitFee
        accountId = accountId or self.account.get('positionId')
        if not accountId:
            raise Exception(
                'No accountId provided' +
                'please call get_account_v2()'
            )

        if not self.configV2:
            raise Exception(
                'No config provided' +
                'please call configs_v2()'
            )
        symbolData = {}
        currency = {}
        config = {}
        if symbol.__contains__("USDT"):
            config = self.usdtConfigV2
        elif symbol.__contains__("USDC"):
            config = self.usdcConfigV2
        for k, v in enumerate(config.get('perpetualContract')):
            if v.get('symbol') == symbol:
                symbolData = v
        for k, v2 in enumerate(config.get('currency')):
            if v2.get('id') == symbolData.get('settleCurrencyId'):
                currency = v2

        if symbolData is not None :
            number = decimal.Decimal(price) / decimal.Decimal(symbolData.get('tickSize'))
            if number > int(number):
                raise Exception(
                    'the price must Multiple of tickSize'
                )

        order_to_sign = SignableOrder(
            position_id=accountId,
            client_id=clientId,
            market=symbol,
            side=side,
            human_size=size,
            human_price=price,
            limit_fee=limitFeeRate,
            expiration_epoch_seconds=expirationEpochSeconds,
            synthetic_resolution=symbolData.get('starkExResolution'),
            synthetic_id=symbolData.get('starkExSyntheticAssetId'),
            collateral_id=currency.get('starkExAssetId'),
        )
        signature = order_to_sign.sign(self.stark_private_key)

        if side == ORDER_SIDE_BUY:
            human_cost = DECIMAL_CONTEXT_ROUND_UP.multiply(
                decimal.Decimal(size),
                decimal.Decimal(price)
            )
            fee = DECIMAL_CONTEXT_ROUND_UP.multiply(human_cost, decimal.Decimal(limitFeeRate))
        else:
            human_cost = DECIMAL_CONTEXT_ROUND_DOWN.multiply(
                decimal.Decimal(size),
                decimal.Decimal(price)
            )
            fee = DECIMAL_CONTEXT_ROUND_DOWN.multiply(human_cost, decimal.Decimal(limitFeeRate))

        limit_fee_rounded = DECIMAL_CONTEXT_ROUND_UP.quantize(
            decimal.Decimal(fee),
            decimal.Decimal(currency.get('stepSize')), )
        expirationEpoch = math.ceil(
            float(expirationEpochSeconds) / ONE_HOUR_IN_SECONDS,
            ) + ORDER_SIGNATURE_EXPIRATION_BUFFER_HOURS

        sl_limit_fee_rounded = None
        slSignature = None
        slTriggerPriceType = None
        slExpiration = None
        tp_limit_fee_rounded = None
        tpSignature = None
        tpTriggerPriceType = None
        tpExpiration = None

        if isOpenTpslOrder == True:
            if isSetOpenSl == True:
                slTriggerPriceType = triggerPriceType
                slExpiration = expirationEpoch * 3600 * 1000
                slClientId = slClientId or random_client_id()
                slOrder_to_sign = SignableOrder(
                    position_id=accountId,
                    client_id=slClientId,
                    market=symbol,
                    side=slSide,
                    human_size=slSize,
                    human_price=slPrice,
                    limit_fee=limitFeeRate,
                    expiration_epoch_seconds=expirationEpochSeconds,
                    synthetic_resolution=symbolData.get('starkExResolution'),
                    synthetic_id=symbolData.get('starkExSyntheticAssetId'),
                    collateral_id=currency.get('starkExAssetId'),
                )
                slSignature = slOrder_to_sign.sign(self.stark_private_key)

                if slSide == ORDER_SIDE_BUY:
                    slHuman_cost = DECIMAL_CONTEXT_ROUND_UP.multiply(
                        decimal.Decimal(slSize),
                        decimal.Decimal(slPrice)
                    )
                    slFee = DECIMAL_CONTEXT_ROUND_UP.multiply(slHuman_cost, decimal.Decimal(limitFeeRate))
                else:
                    slHuman_cost = DECIMAL_CONTEXT_ROUND_DOWN.multiply(
                        decimal.Decimal(slSize),
                        decimal.Decimal(slPrice)
                    )
                    slFee = DECIMAL_CONTEXT_ROUND_DOWN.multiply(slHuman_cost, decimal.Decimal(limitFeeRate))

                sl_limit_fee_rounded = DECIMAL_CONTEXT_ROUND_UP.quantize(
                    decimal.Decimal(slFee),
                    decimal.Decimal(currency.get('stepSize')), )

            if isSetOpenTp == True:
                tpTriggerPriceType = triggerPriceType
                tpExpiration = expirationEpoch * 3600 * 1000
                tpClientId = tpClientId or random_client_id()
                tpOrder_to_sign = SignableOrder(
                    position_id=accountId,
                    client_id=tpClientId,
                    market=symbol,
                    side=tpSide,
                    human_size=tpSize,
                    human_price=tpPrice,
                    limit_fee=limitFeeRate,
                    expiration_epoch_seconds=expirationEpochSeconds,
                    synthetic_resolution=symbolData.get('starkExResolution'),
                    synthetic_id=symbolData.get('starkExSyntheticAssetId'),
                    collateral_id=currency.get('starkExAssetId'),
                )
                tpSignature = tpOrder_to_sign.sign(self.stark_private_key)

                if tpSide == ORDER_SIDE_BUY:
                    tpHuman_cost = DECIMAL_CONTEXT_ROUND_UP.multiply(
                        decimal.Decimal(tpSize),
                        decimal.Decimal(tpPrice)
                    )
                    tpFee = DECIMAL_CONTEXT_ROUND_UP.multiply(tpHuman_cost, decimal.Decimal(limitFeeRate))
                else:
                    tpHuman_cost = DECIMAL_CONTEXT_ROUND_DOWN.multiply(
                        decimal.Decimal(tpSize),
                        decimal.Decimal(tpPrice)
                    )
                    tpFee = DECIMAL_CONTEXT_ROUND_DOWN.multiply(tpHuman_cost, decimal.Decimal(limitFeeRate))

                tp_limit_fee_rounded = DECIMAL_CONTEXT_ROUND_UP.quantize(
                    decimal.Decimal(tpFee),
                    decimal.Decimal(currency.get('stepSize')), )

        order = {
            'symbol': symbol,
            'side': side,
            'type': type,
            'timeInForce': timeInForce,
            'size': size,
            'price': price,
            'limitFee': str(limit_fee_rounded),
            'expiration': expirationEpoch * 3600 * 1000,
            'triggerPrice': triggerPrice,
            'triggerPriceType': triggerPriceType,
            'trailingPercent': trailingPercent,
            'clientId': clientId,
            'signature': signature,
            'reduceOnly': reduceOnly,
            'isPositionTpsl': isPositionTpsl,
            'isOpenTpslOrder': isOpenTpslOrder,
            'isSetOpenSl': isSetOpenSl,
            'isSetOpenTp': isSetOpenTp,
            'slClientOrderId': slClientId,
            'slPrice': slPrice,
            'slSide': slSide,
            'slSize': slSize,
            'slTriggerPrice': slTriggerPrice,
            'slTriggerPriceType': slTriggerPriceType,
            'slExpiration': slExpiration,
            'slLimitFee': str(sl_limit_fee_rounded),
            'slSignature': slSignature,
            'tpClientOrderId': tpClientId,
            'tpPrice': tpPrice,
            'tpSide': tpSide,
            'tpSize': tpSize,
            'tpTriggerPrice': tpTriggerPrice,
            'tpTriggerPriceType': tpTriggerPriceType,
            'tpExpiration': tpExpiration,
            'tpLimitFee': str(tp_limit_fee_rounded),
            'tpSignature': tpSignature,
            'sourceFlag': sourceFlag,
            'brokerId':brokerId,
        }

        path = URL_SUFFIX + "/v2/create-order"
        return self._post(
            endpoint=path,
            data=order
        )

    def create_withdrawal(self,
                          amount,
                          asset,
                          accountId=None,
                          clientId=None,
                          expiration=None,
                          expirationEpochSeconds=None,
                          ethAddress=None,
                          signature=None, ):
        """"
        POST  create-withdrawal-to-address.
       :returns: Request results as dictionary.
       """

        clientId = clientId or random_client_id()
        if not self.stark_private_key:
            raise Exception(
                'No signature provided and client was not ' +
                'initialized with stark_private_key'
            )

        if bool(expiration) == bool(expirationEpochSeconds):
            raise ValueError(
                'Exactly one of expiration and expiration_epoch_seconds must '
                'be specified',
            )
        expiration = expiration or epoch_seconds_to_iso(
            expirationEpochSeconds,
        )
        expirationEpochSeconds = (
                expirationEpochSeconds or iso_to_epoch_seconds(expiration)
        )

        ethAddress = ethAddress or self.account.get('ethereumAddress')
        if not ethAddress:
            raise Exception(
                'No ethAddress provided' +
                'please call get_user()'
            )

        accountId = accountId or self.account.get('positionId')
        if not accountId:
            raise Exception(
                'No accountId provided' +
                'please call get_account()'
            )

        if not self.config:
            raise Exception(
                'No config provided' +
                'please call configs()'
            )

        currency = {}

        for k, v in enumerate(self.config.get('currency')):
            if v.get('id') == asset:
                currency = v
        withdraw_to_sign = SignableWithdrawal(
            network_id=self.network_id,
            position_id=accountId,
            client_id=clientId,
            human_amount=amount,
            expiration_epoch_seconds=expirationEpochSeconds,
            eth_address=ethAddress,
            collateral_id=currency.get('starkExAssetId'),
        )
        signature = withdraw_to_sign.sign(self.stark_private_key)

        expirationEpoch = math.ceil(
            float(expirationEpochSeconds) / ONE_HOUR_IN_SECONDS,
        ) + ORDER_SIGNATURE_EXPIRATION_BUFFER_HOURS

        withdraw = {
            'amount': amount,
            'asset': asset,
            'expiration': expirationEpoch * 3600 * 1000,
            'ethAddress': ethAddress,
            'clientId': clientId,
            'signature': signature,
        }

        path = URL_SUFFIX + "/v1/create-withdrawal-to-address"
        return self._post(
            endpoint=path,
            data=withdraw
        )

    def create_withdrawal_v2(self,
                          amount,
                          asset,
                          accountId=None,
                          clientId=None,
                          expiration=None,
                          expirationEpochSeconds=None,
                          ethAddress=None,
                          signature=None, ):
        """"
        POST  create-withdrawal-to-address.
       :returns: Request results as dictionary.
       """

        clientId = clientId or random_client_id()
        if not self.stark_private_key:
            raise Exception(
                'No signature provided and client was not ' +
                'initialized with stark_private_key'
            )

        if bool(expiration) == bool(expirationEpochSeconds):
            raise ValueError(
                'Exactly one of expiration and expiration_epoch_seconds must '
                'be specified',
            )
        expiration = expiration or epoch_seconds_to_iso(
            expirationEpochSeconds,
        )
        expirationEpochSeconds = (
                expirationEpochSeconds or iso_to_epoch_seconds(expiration)
        )

        ethAddress = ethAddress or self.account.get('ethereumAddress')
        if not ethAddress:
            raise Exception(
                'No ethAddress provided' +
                'please call get_user()'
            )

        accountId = accountId or self.account.get('positionId')
        if not accountId:
            raise Exception(
                'No accountId provided' +
                'please call get_account()'
            )

        currency = {}
        config = {}
        if asset.__contains__("USDT"):
            config = self.usdtConfigV2
        elif asset.__contains__("USDC"):
            config = self.usdcConfigV2


        for k, v in enumerate(config.get('currency')):
            if v.get('id') == asset:
                currency = v
        withdraw_to_sign = SignableWithdrawal(
            network_id=self.network_id,
            position_id=accountId,
            client_id=clientId,
            human_amount=amount,
            expiration_epoch_seconds=expirationEpochSeconds,
            eth_address=ethAddress,
            collateral_id=currency.get('starkExAssetId'),
        )
        signature = withdraw_to_sign.sign(self.stark_private_key)

        expirationEpoch = math.ceil(
            float(expirationEpochSeconds) / ONE_HOUR_IN_SECONDS,
            ) + ORDER_SIGNATURE_EXPIRATION_BUFFER_HOURS

        withdraw = {
            'amount': amount,
            'asset': asset,
            'expiration': expirationEpoch * 3600 * 1000,
            'ethAddress': ethAddress,
            'clientId': clientId,
            'signature': signature,
        }

        path = URL_SUFFIX + "/v2/create-withdrawal-to-address"
        return self._post(
            endpoint=path,
            data=withdraw
        )

    def fast_withdrawal(self,
                        amount,
                        asset,
                        accountId=None,
                        clientId=None,
                        expiration=None,
                        expirationEpochSeconds=None,
                        ethAddress=None,
                        erc20Address=None,
                        fee=None,
                        lpAccountId=None,
                        signature=None, ):
        """"
        POST  fast_withdrawal.
       :returns: Request results as dictionary.
       """

        clientId = clientId or random_client_id()
        if not self.stark_private_key:
            raise Exception(
                'No signature provided and client was not ' +
                'initialized with stark_private_key'
            )

        if bool(expiration) == bool(expirationEpochSeconds):
            raise ValueError(
                'Exactly one of expiration and expiration_epoch_seconds must '
                'be specified',
            )
        expiration = expiration or epoch_seconds_to_iso(
            expirationEpochSeconds,
        )
        expirationEpochSeconds = (
                expirationEpochSeconds or iso_to_epoch_seconds(expiration)
        )

        accountId = accountId or self.account.get('positionId')
        if not accountId:
            raise Exception(
                'No accountId provided' +
                'please call get_account()'
            )

        ethAddress = ethAddress or self.user.get('ethereumAddress') or self.account.get('ethereumAddress')
        if not ethAddress:
            raise Exception(
                'No ethAddress provided' +
                'please call get_user()'
            )

        if not self.config:
            raise Exception(
                'No config provided' +
                'please call configs()'
            )

        currency = {}
        for k, v in enumerate(self.config.get('currency')):
            if v.get('id') == asset:
                currency = v

        token = {}
        for k, v1 in enumerate(self.config.get('multiChain').get('chains')):
            if v1.get('chainId') == self.network_id:
                for _, v2 in enumerate(v1.get('tokens')):
                    if v2.get('token') == asset:
                        if v2.get('withdrawEnable') == True:
                            token = v2

        fact = get_transfer_erc20_fact(
            recipient=ethAddress,
            token_decimals=token.get('decimals'),
            human_amount=amount,
            token_address=(
                token.get('tokenAddress')
            ),
            salt=nonce_from_client_id(clientId),
        )

        totalAmount = decimal.Decimal(amount) + decimal.Decimal(fee)
        transfer_to_sign = SignableConditionalTransfer(
            network_id=self.network_id,
            sender_position_id=accountId,
            receiver_position_id=self.config.get('global').get('fastWithdrawAccountId'),
            receiver_public_key=self.config.get('global').get('fastWithdrawL2Key'),
            fact_registry_address=self.config.get('global').get('fastWithdrawFactRegisterAddress'),
            fact=fact,
            human_amount=str(totalAmount),
            client_id=clientId,
            expiration_epoch_seconds=expirationEpochSeconds,
            collateral_id=currency.get('starkExAssetId')
        )

        signature = transfer_to_sign.sign(self.stark_private_key)

        expirationEpoch = math.ceil(
            float(expirationEpochSeconds) / ONE_HOUR_IN_SECONDS,
        ) + ORDER_SIGNATURE_EXPIRATION_BUFFER_HOURS

        withdraw = {
            'amount': amount,
            'asset': asset,
            'expiration': expirationEpoch * 3600 * 1000,
            'ethAddress': ethAddress,
            'clientId': clientId,
            'signature': signature,
            'erc20Address': token.get('tokenAddress'),
            'fee': fee,
            'lpAccountId': self.config.get('global').get('fastWithdrawAccountId'),
            'chainId': self.network_id
        }

        path = URL_SUFFIX + "/v1/fast-withdraw"
        return self._post(
            endpoint=path,
            data=withdraw
        )

    def fast_withdrawal_v2(self,
                        amount,
                        asset,
                        accountId=None,
                        clientId=None,
                        expiration=None,
                        expirationEpochSeconds=None,
                        ethAddress=None,
                        erc20Address=None,
                        fee=None,
                        lpAccountId=None,
                        signature=None, ):
        """"
        POST  fast_withdrawal.
       :returns: Request results as dictionary.
       """

        clientId = clientId or random_client_id()
        if not self.stark_private_key:
            raise Exception(
                'No signature provided and client was not ' +
                'initialized with stark_private_key'
            )

        if bool(expiration) == bool(expirationEpochSeconds):
            raise ValueError(
                'Exactly one of expiration and expiration_epoch_seconds must '
                'be specified',
            )
        expiration = expiration or epoch_seconds_to_iso(
            expirationEpochSeconds,
        )
        expirationEpochSeconds = (
                expirationEpochSeconds or iso_to_epoch_seconds(expiration)
        )

        accountId = accountId or self.account.get('positionId')
        if not accountId:
            raise Exception(
                'No accountId provided' +
                'please call get_account()'
            )

        ethAddress = ethAddress or self.user.get('ethereumAddress') or self.account.get('ethereumAddress')
        if not ethAddress:
            raise Exception(
                'No ethAddress provided' +
                'please call get_user()'
            )

        currency = {}
        config = {}
        if asset.__contains__("USDT"):
            config = self.usdtConfigV2
        elif asset.__contains__("USDC"):
            config = self.usdcConfigV2

        for k, v in enumerate(config.get('currency')):
            if v.get('id') == asset:
                currency = v

        token = {}
        for k, v1 in enumerate(config.get('multiChain').get('chains')):
            if v1.get('chainId') == self.network_id:
                for _, v2 in enumerate(v1.get('tokens')):
                    if v2.get('token') == asset:
                        if v2.get('withdrawEnable') == True:
                            token = v2

        fact = get_transfer_erc20_fact(
            recipient=ethAddress,
            token_decimals=token.get('decimals'),
            human_amount=amount,
            token_address=(
                token.get('tokenAddress')
            ),
            salt=nonce_from_client_id(clientId),
        )

        totalAmount = decimal.Decimal(amount) + decimal.Decimal(fee)
        transfer_to_sign = SignableConditionalTransfer(
            network_id=self.network_id,
            sender_position_id=accountId,
            receiver_position_id=config.get('global').get('fastWithdrawAccountId'),
            receiver_public_key=config.get('global').get('fastWithdrawL2Key'),
            fact_registry_address=config.get('global').get('fastWithdrawFactRegisterAddress'),
            fact=fact,
            human_amount=str(totalAmount),
            client_id=clientId,
            expiration_epoch_seconds=expirationEpochSeconds,
            collateral_id=currency.get('starkExAssetId')
        )

        signature = transfer_to_sign.sign(self.stark_private_key)

        expirationEpoch = math.ceil(
            float(expirationEpochSeconds) / ONE_HOUR_IN_SECONDS,
            ) + ORDER_SIGNATURE_EXPIRATION_BUFFER_HOURS

        withdraw = {
            'amount': amount,
            'asset': asset,
            'expiration': expirationEpoch * 3600 * 1000,
            'ethAddress': ethAddress,
            'clientId': clientId,
            'signature': signature,
            'erc20Address': token.get('tokenAddress'),
            'fee': fee,
            'lpAccountId': config.get('global').get('fastWithdrawAccountId'),
            'chainId': self.network_id
        }

        path = URL_SUFFIX + "/v2/fast-withdraw"
        return self._post(
            endpoint=path,
            data=withdraw
        )

    def cross_chain_withdraw(self,
                             amount,
                             asset,
                             chainId,
                             accountId=None,
                             clientId=None,
                             expiration=None,
                             expirationEpochSeconds=None,
                             erc20Address=None,
                             fee=None,
                             lpAccountId=None,
                             signature=None, ):
        """"
        POST  cross_chain_withdraw.
       :returns: Request results as dictionary.
       """
        clientId = clientId or random_client_id()
        if not self.stark_private_key:
            raise Exception(
                'No signature provided and client was not ' +
                'initialized with stark_private_key'
            )

        if bool(expiration) == bool(expirationEpochSeconds):
            raise ValueError(
                'Exactly one of expiration and expiration_epoch_seconds must '
                'be specified',
            )
        expiration = expiration or epoch_seconds_to_iso(
            expirationEpochSeconds,
        )
        expirationEpochSeconds = (
                expirationEpochSeconds or iso_to_epoch_seconds(expiration)
        )

        accountId = accountId or self.account.get('positionId')
        if not accountId:
            raise Exception(
                'No accountId provided' +
                'please call gett_account()'
            )

        if not self.config:
            raise Exception(
                'No config provided' +
                'please call configs()'
            )

        currency = {}
        for k, v in enumerate(self.config.get('currency')):
            if v.get('id') == asset:
                currency = v

        token = {}
        for k, v1 in enumerate(self.config.get('multiChain').get('chains')):
            if v1.get('chainId') == int(chainId):
                for _, v2 in enumerate(v1.get('tokens')):
                    if v2.get('token') == asset:
                        if v2.get('withdrawEnable') == True:
                            token = v2

        totalAmount = decimal.Decimal(amount) + decimal.Decimal(fee)
        transfer_to_sign = SignableTransfer(
            network_id=chainId,
            sender_position_id=accountId,
            receiver_position_id=self.config.get('global').get('crossChainAccountId'),
            receiver_public_key=self.config.get('global').get('crossChainL2Key'),
            human_amount=str(totalAmount),
            client_id=clientId,
            expiration_epoch_seconds=expirationEpochSeconds,
            collateral_id=currency.get('starkExAssetId')
        )
        signature = transfer_to_sign.sign(self.stark_private_key)

        expirationEpoch = math.ceil(
            float(expirationEpochSeconds) / ONE_HOUR_IN_SECONDS,
        ) + ORDER_SIGNATURE_EXPIRATION_BUFFER_HOURS

        withdraw = {
            'amount': amount,
            'asset': asset,
            'expiration': expirationEpoch * 3600 * 1000,
            'clientId': clientId,
            'signature': signature,
            'erc20Address': token.get('tokenAddress'),
            'fee': fee,
            'lpAccountId': self.config.get('global').get('crossChainAccountId'),
            'chainId': chainId
        }
        path = URL_SUFFIX + "/v1/cross-chain-withdraw"
        return self._post(
            endpoint=path,
            data=withdraw
        )

    def cross_chain_withdraw_v2(self,
                             amount,
                             asset,
                             chainId,
                             accountId=None,
                             clientId=None,
                             expiration=None,
                             expirationEpochSeconds=None,
                             erc20Address=None,
                             fee=None,
                             lpAccountId=None,
                             signature=None, ):
        """"
        POST  cross_chain_withdraw.
       :returns: Request results as dictionary.
       """
        clientId = clientId or random_client_id()
        if not self.stark_private_key:
            raise Exception(
                'No signature provided and client was not ' +
                'initialized with stark_private_key'
            )

        if bool(expiration) == bool(expirationEpochSeconds):
            raise ValueError(
                'Exactly one of expiration and expiration_epoch_seconds must '
                'be specified',
            )
        expiration = expiration or epoch_seconds_to_iso(
            expirationEpochSeconds,
        )
        expirationEpochSeconds = (
                expirationEpochSeconds or iso_to_epoch_seconds(expiration)
        )

        accountId = accountId or self.account.get('positionId')
        if not accountId:
            raise Exception(
                'No accountId provided' +
                'please call gett_account()'
            )

        currency = {}
        config = {}
        if asset.__contains__("USDT"):
            config = self.usdtConfigV2
        elif asset.__contains__("USDC"):
            config = self.usdcConfigV2

        for k, v in enumerate(config.get('currency')):
            if v.get('id') == asset:
                currency = v

        token = {}
        for k, v1 in enumerate(config.get('multiChain').get('chains')):
            if v1.get('chainId') == int(chainId):
                for _, v2 in enumerate(v1.get('tokens')):
                    if v2.get('token') == asset:
                        if v2.get('withdrawEnable') == True:
                            token = v2

        totalAmount = decimal.Decimal(amount) + decimal.Decimal(fee)
        transfer_to_sign = SignableTransfer(
            network_id=chainId,
            sender_position_id=accountId,
            receiver_position_id=config.get('global').get('crossChainAccountId'),
            receiver_public_key=config.get('global').get('crossChainL2Key'),
            human_amount=str(totalAmount),
            client_id=clientId,
            expiration_epoch_seconds=expirationEpochSeconds,
            collateral_id=currency.get('starkExAssetId')
        )
        signature = transfer_to_sign.sign(self.stark_private_key)

        expirationEpoch = math.ceil(
            float(expirationEpochSeconds) / ONE_HOUR_IN_SECONDS,
            ) + ORDER_SIGNATURE_EXPIRATION_BUFFER_HOURS

        withdraw = {
            'amount': amount,
            'asset': asset,
            'expiration': expirationEpoch * 3600 * 1000,
            'clientId': clientId,
            'signature': signature,
            'erc20Address': token.get('tokenAddress'),
            'fee': fee,
            'lpAccountId': config.get('global').get('crossChainAccountId'),
            'chainId': chainId
        }
        path = URL_SUFFIX + "/v2/cross-chain-withdraw"
        return self._post(
            endpoint=path,
            data=withdraw
        )



import os
import sys
import time

from apexpro.http_private_v3 import HttpPrivate_v3

root_path = os.path.abspath(__file__)
root_path = '/'.join(root_path.split('/')[:-2])
sys.path.append(root_path)

from apexpro.constants import APEX_OMNI_HTTP_MAIN, NETWORKID_OMNI_MAIN_ARB

print("Hello, Apex Omni")
priKey = "your eth private key"

client = HttpPrivate_v3(APEX_OMNI_HTTP_MAIN, network_id=NETWORKID_OMNI_MAIN_ARB, eth_private_key=priKey)
configs = client.configs_v3()

zkKeys = client.derive_zk_key(client.default_address)

nonceRes = client.generate_nonce_v3(refresh="false", l2Key=zkKeys['l2Key'],ethAddress=client.default_address, chainId=NETWORKID_OMNI_MAIN_ARB)

regRes = client.register_user_v3(nonce=nonceRes['data']['nonce'],l2Key=zkKeys['l2Key'], seeds=zkKeys['seeds'],ethereum_address=client.default_address,
                                 eth_mul_address="your mul eth address", isLpAccount=True)
print(regRes)
key = regRes['data']['apiKey']['key']
secret = regRes['data']['apiKey']['secret']
passphrase = regRes['data']['apiKey']['passphrase']

time.sleep(10)
accountRes = client.get_account_v3()
print(accountRes)

#back stark_key_pair, apiKey,and accountId for private Api or create-oreder or withdraw

seeds = zkKeys.get('seeds')
l2Key = zkKeys.get('l2Key')
pubKeyHash = zkKeys.get('pubKeyHash')

#back zkKeys, seeds, l2Key and pubKeyHash for register_step2
print(zkKeys)
print(regRes['data']['account']['id'])
print(regRes['data']['apiKey'])

print("end, Apex Omni")






import os
import sys

root_path = os.path.abspath(__file__)
root_path = '/'.join(root_path.split('/')[:-2])
sys.path.append(root_path)

from apexpro.http_private import HttpPrivate
from apexpro.constants import APEX_HTTP_TEST, NETWORKID_TEST, APEX_HTTP_MAIN, NETWORKID_MAIN

print("Hello, Apexpro")
priKey = "your eth private key"

client = HttpPrivate(APEX_HTTP_MAIN, network_id=NETWORKID_MAIN, eth_private_key=priKey)
configs = client.configs()

stark_key_pair_with_y_coordinate = client.derive_stark_key(client.default_address)

nonceRes = client.generate_nonce(starkKey=stark_key_pair_with_y_coordinate['public_key'],ethAddress=client.default_address,chainId=NETWORKID_MAIN)

#api_key = client.recover_api_key_credentials(nonce=nonceRes['data']['nonce'], ethereum_address=client.default_address)
#print(api_key)

regRes = client.register_user(nonce=nonceRes['data']['nonce'],starkKey=stark_key_pair_with_y_coordinate['public_key'],stark_public_key_y_coordinate=stark_key_pair_with_y_coordinate['public_key_y_coordinate'],ethereum_address=client.default_address,
                              eth_mul_address="your mul eth address")
key = regRes['data']['apiKey']['key']
secret = regRes['data']['apiKey']['secret']
passphrase = regRes['data']['apiKey']['passphrase']

#back stark_key_pair, apiKey,and accountId for private Api or create-oreder or withdraw
print(stark_key_pair_with_y_coordinate)
print(regRes['data']['account']['positionId'])
print(regRes['data']['apiKey'])
print("end, Apexpro")






import os
import sys

root_path = os.path.abspath(__file__)
root_path = '/'.join(root_path.split('/')[:-2])
sys.path.append(root_path)


from apexpro.constants import APEX_OMNI_HTTP_MAIN
from apexpro.http_public import HttpPublic

print("Hello, ApexOmni")

# APEX_OMNI_HTTP_MAIN for mainnet, APEX_OMNI_HTTP_TEST for testnet
client = HttpPublic(APEX_OMNI_HTTP_MAIN)
print(client.klines_v3(symbol="ETHUSDT",interval=5,start=1718358480, end=1718950620, limit=5))
print(client.configs_v3())
print(client.depth_v3(symbol="BTCUSDT"))
print(client.trades_v3(symbol="BTCUSDT"))
print(client.klines_v3(symbol="BTCUSDT",interval="15"))
print(client.ticker_v3(symbol="BTCUSDT"))
print(client.history_funding_v3(symbol="BTC-USDT"))




from time import sleep

from apexpro.constants import APEX_WS_TEST, APEX_HTTP_TEST
from apexpro.http_public import HttpPublic
from apexpro.websocket_api import WebSocket

key = 'your apiKey-key from register'
secret = 'your apiKey-secret from register'
passphrase = 'your apiKey-passphrase from register'

# Connect with authentication!
ws_client = WebSocket(
    endpoint=APEX_WS_TEST,
    api_key_credentials={'key': key, 'secret': secret, 'passphrase': passphrase},
)
client = HttpPublic(APEX_HTTP_TEST)
symbol_list = client.configs().get("data").get("perpetualContract")
print("symbol_list:", symbol_list)

positions = {}
wallets = {}
orders = {}

def handle_account(message):
    print("all account: ", message)
    if message.get("contents") is not None and message.get("contents").get("positions") is not None:
        positions = message.get("contents").get("positions")
        print("positions: ", positions)

def h2(message):
    print("ticker: ", message)

#ws_client.depth_stream(h1,'BTCUSDC',25)
ws_client.ticker_stream(h2,'All')
#ws_client.trade_stream(h3,'BTCUSDC')
#ws_client.klines_stream(h4,'BTCUSDC',1)
ws_client.account_info_stream(handle_account)



while True:
    # Run your main trading logic here.
    sleep(1)



from time import sleep

from apexpro.constants import APEX_OMNI_WS_MAIN
from apexpro.websocket_api import WebSocket

key = 'your apiKey-key from register V3'
secret = 'your apiKey-secret from register  V3'
passphrase = 'your apiKey-passphrase from register  V3'


# Connect with authentication!
# APEX_OMNI_WS_MAIN for mainnet, APEX_OMNI_WS_TEST for testnet
ws_client = WebSocket(
    endpoint=APEX_OMNI_WS_MAIN,
    api_key_credentials={'key': key, 'secret': secret, 'passphrase': passphrase},
)

def handle_account(message):
    print(message)
    contents_data = message["contents"]
    print(len(contents_data))

def h1(message):
    print(1, message)
def h2(message):
    print(2, message)
def h3(message):
    print(3, message)
def h4(message):
    print(4, message)

#ws_client.depth_stream(h1,'BTCUSDT',25)
#ws_client.ticker_stream(h2,'BTCUSDT')
ws_client.trade_stream(h3,'BTCUSDT')
ws_client.klines_stream(h4,'BTCUSDT',1)
ws_client.account_info_stream_v3(handle_account)


while True:
    # Run your main trading logic here.
    sleep(1)



import decimal
import time
from threading import Timer

from apexpro.constants import APEX_WS_TEST, APEX_HTTP_TEST, NETWORKID_TEST
from apexpro.http_private import HttpPrivate
from apexpro.http_public import HttpPublic
from apexpro.websocket_api import WebSocket

key = 'your apiKey-key from register'
secret = 'your apiKey-secret from register'
passphrase = 'your apiKey-passphrase from register'

ticker_time = 10

# Connect with authentication!
ws_client = WebSocket(
    endpoint=APEX_WS_TEST
)
client = HttpPrivate(APEX_HTTP_TEST, network_id=NETWORKID_TEST,
                     api_key_credentials={'key': key, 'secret': secret, 'passphrase': passphrase})

symbol_list = client.configs().get("data").get("perpetualContract")
print("symbol_list:", symbol_list)


wallets = {}
openPositions = {}
orders = {}

allTickerPrice = []

def all_ticker(message):
    global allTickerPrice
    print("all_ticker: ", message)
    newAllTickerPrice = message.get("data")

    if len(wallets) == 0:
        allTickerPrice = newAllTickerPrice
        handle_account()
        return

    if newAllTickerPrice is not None:
        for k, position in enumerate(openPositions):
            oldPrice = decimal.Decimal(get_symbol_price(position.get('symbol')))
            symbolData = get_symbol_config(position.get('symbol'))
            for k, v in enumerate(newAllTickerPrice):
                if v['s'] == symbolData.get('symbol') or v['s'] == symbolData.get('crossSymbolName') or v['s'] == symbolData.get(
                'symbolDisplayName'):
                    if v.get("op") is not None:
                        newPrice =  decimal.Decimal(v.get("op"))
                        if oldPrice != newPrice:
                            print(v['s'] + "  oldPrice: " + str(oldPrice) + "  newPrice: " + str(newPrice))
                            handle_account()
                            allTickerPrice = newAllTickerPrice
                            return
        allTickerPrice = newAllTickerPrice


ws_client.all_ticker_stream(all_ticker)


def handle_account():
    print("current time:", time.time())
    global client, wallets, openPositions, orders
    account = client.get_account()
    if account.get("data") is not None and account.get("data").get("wallets") is not None:
        wallets = account.get("data").get("wallets")
    if account.get("data") is not None and account.get("data").get("openPositions") is not None:
        openPositions = account.get("data").get("openPositions")

    get_orders = client.open_orders()
    if get_orders.get("data") is not None:
        orders = get_orders.get("data")

    totalAccountValue = decimal.Decimal('0.0')
    for k, wallet in enumerate(wallets):
        if wallet.get("asset") == "USDC":
            totalAccountValue = decimal.Decimal(wallet.get('balance')) - decimal.Decimal(wallet.get('pendingWithdrawAmount')) - decimal.Decimal(wallet.get('pendingTransferOutAmount'))
    for k, position in enumerate(openPositions):
        positionValue = decimal.Decimal(position.get('size')) * decimal.Decimal(get_symbol_price(position.get('symbol')))
        if position.get('side') == "SHORT":
            positionValue = positionValue * decimal.Decimal(-1)
        totalAccountValue = totalAccountValue + positionValue

    print("totalAccountValue is :" + str(totalAccountValue))

    totalInitialMarginRequirement = decimal.Decimal('0.0')
    for k, position in enumerate(openPositions):
        totalInitialMarginRequirement = totalInitialMarginRequirement + decimal.Decimal(position.get('entryPrice')) * decimal.Decimal(position.get('size')) * decimal.Decimal(get_symbol_config(position.get('symbol')).get('initialMarginRate'))

    orderTotalInitialMarginRequirement = decimal.Decimal('0.0')
    for k, order in enumerate(orders):
        orderTotalInitialMarginRequirement = orderTotalInitialMarginRequirement + decimal.Decimal(order.get('price')) * decimal.Decimal(order.get('size')) * decimal.Decimal(get_symbol_config(position.get('symbol')).get('initialMarginRate'))
        orderTotalInitialMarginRequirement = orderTotalInitialMarginRequirement + decimal.Decimal(order.get('price')) * decimal.Decimal(order.get('size')) * decimal.Decimal(account.get("data").get('takerFeeRate'))


    availableValue =  totalAccountValue - totalInitialMarginRequirement - orderTotalInitialMarginRequirement

    print("availableValue is :" + str(availableValue))


    totalMaintenanceMarginRequirement = decimal.Decimal('0.0')
    for k, position in enumerate(openPositions):
        totalMaintenanceMarginRequirement = totalMaintenanceMarginRequirement + decimal.Decimal(position.get('size')) * decimal.Decimal(get_symbol_price(position.get('symbol'))) * decimal.Decimal(get_symbol_config(position.get('symbol')).get('maintenanceMarginRate'))


    print("totalMaintenanceMarginRequirement is :" + str(totalMaintenanceMarginRequirement))



def get_symbol_config(symbol):
    for k, v in enumerate(symbol_list):
        if v.get('symbol') == symbol or v.get('crossSymbolName') == symbol or v.get('symbolDisplayName') == symbol:
            return v


def get_symbol_price(symbol):
    global allTickerPrice
    symbolData = get_symbol_config(symbol)
    for k, v in enumerate(allTickerPrice):
        if v['s'] == symbolData.get('symbol') or v['s'] == symbolData.get('crossSymbolName') or v['s'] == symbolData.get(
                'symbolDisplayName'):
            return v.get("op")


while True:
    # Run your main trading logic here.
    time.sleep(2)
    timer = Timer(ticker_time, handle_account)
    timer.start()
    timer.join()



import os
import sys

from apexpro.helpers.request_helpers import calc_bind_owner_key_sig_hash, starkex_sign, starkex_verify

root_path = os.path.abspath(__file__)
root_path = '/'.join(root_path.split('/')[:-2])
sys.path.append(root_path)


print("Hello, Apexpro")

hash = calc_bind_owner_key_sig_hash('your stark_key_pair pubic key', "your eth address")
print(hash.hex())

signature = starkex_sign(hash, 'your stark_key_pair private key')
print("signature:" + signature)

bVerify = starkex_verify(hash, signature, 'your stark_key_pair pubic key')
print("Verify:" +str( bVerify))






import json
import random
import time
from datetime import datetime

import dateutil.parser as dp
from web3 import Web3

from apexpro.eth_signing.util import strip_hex_prefix
from apexpro.starkex.helpers import serialize_signature, deserialize_signature, int_to_hex_32
from apexpro.starkex.starkex_resources.proxy import sign, verify, get_hash
from apexpro.starkex.starkex_resources.python_signature import inv_mod_curve_size


def generate_query_path(url, params):
    entries = params.items()
    if not entries:
        return url

    paramsString = '&'.join('{key}={value}'.format(
        key=x[0], value=x[1]) for x in entries if x[1] is not None)
    if paramsString:
        return url + '?' + paramsString

    return url


def json_stringify(data):
    return json.dumps(data, separators=(',', ':'))

def json_msg_stringify(data):
    return json.dumps(data, separators=(',', ': '), indent=2)


def random_client_id():
    return str(int(float(str(random.random())[2:])))


def generate_now_iso():
    return datetime.utcnow().strftime(
        '%Y-%m-%dT%H:%M:%S.%f',
    )[:-3] + 'Z'


def generate_now():
    now = time.time()
    return int(round(now * 1000))


def iso_to_epoch_seconds(iso):
    return dp.parse(iso).timestamp()


def epoch_seconds_to_iso(epoch):
    return datetime.utcfromtimestamp(epoch).strftime(
        '%Y-%m-%dT%H:%M:%S.%f',
    )[:-3] + 'Z'


def remove_nones(original):
    return {k: v for k, v in original.items() if v is not None}


def calc_bind_owner_key_sig_hash(star_key_hex, owner_key):
    data_bytes = "UserRegistration:"
    owner_key_bytes = bytes.fromhex(strip_hex_prefix(owner_key))
    data = Web3.solidityKeccak(
        [
            'string',
            'bytes20',
            'uint256',
        ],
        [
            data_bytes,
            owner_key_bytes,
            int(star_key_hex, 16),
        ]
    )
    print(data.hex())
    return data


def starkex_sign(hash, private_key_hex):
    """Sign the hash of the object using the given private key."""
    EC_ORDER = 3618502788666131213697322783095070105526743751716087489154079457884512865583
    hash_mod = int(hash.hex(), 16) % EC_ORDER
    print("hash_mod:" + int_to_hex_32(hash_mod))
    r, s = sign(hash_mod, int(private_key_hex, 16))
    return serialize_signature(r, s)

def starkex_verify(hash, sign, pub_key):
    EC_ORDER = 3618502788666131213697322783095070105526743751716087489154079457884512865583
    hash_mod = int(hash.hex(), 16) % EC_ORDER
    r, s = deserialize_signature(sign)
    return verify(hash_mod, r, s, int(pub_key, 16))



import os
import sys


root_path = os.path.abspath(__file__)
root_path = '/'.join(root_path.split('/')[:-2])
sys.path.append(root_path)

from apexpro.http_private import HttpPrivate

from apexpro.constants import APEX_HTTP_TEST, NETWORKID_TEST

print("Hello, Apexpro")
# need api_key_credentials={'key': key,'secret': secret, 'passphrase': passphrase} for private api

key = 'your apiKey-key from register'
secret = 'your apiKey-secret from register'
passphrase = 'your apiKey-passphrase from register'


client = HttpPrivate(APEX_HTTP_TEST, network_id=NETWORKID_TEST, api_key_credentials={'key': key,'secret': secret, 'passphrase': passphrase})
configs = client.configs_v2()

#userRes = client.get_user()
#print(userRes)


#modifyUserRes = client.modify_user(username="pythonTest",email="11@aa.com",emailNotifyGeneralEnable="false")
#print(modifyUserRes)

accountRes = client.get_account_v2()
print(accountRes)

fillsRes = client.fills_v2(limit=100,page=0,symbol="BTC-USDC",side="BUY",token="USDC")
print(fillsRes)

transfersRes = client.transfers_v2(limit=100,page=0,currencyId="USDC",chainIds="1,5,13,97")
print(transfersRes)

withdrawListRes = client.withdraw_list_v2(limit=100,page=0,beginTimeInclusive=1651406864000,endTimeExclusive=1657105971171)
print(withdrawListRes)

uncommon_withdraw_feeRes = client.uncommon_withdraw_fee_v2(amount="101000.1",token="USDC", chainId=5)
print(uncommon_withdraw_feeRes)

transfer_limitRes = client.transfer_limit_v2(currencyId="USDC")
print(transfer_limitRes)

fillsRes = client.fills_v2(limit=100,page=0,symbol="BTC-USDC",side="BUY",token="USDC")
print(fillsRes)

deleteOrderRes = client.delete_order_v2(id="123456")
print(deleteOrderRes)

deleteOrderRes = client.delete_order_by_client_order_id_v2(id="123456")
print(deleteOrderRes)

openOrdersRes = client.open_orders_v2(token='USDC')
print(openOrdersRes)

deleteOrdersRes = client.delete_open_orders(symbol="BTC-USDC,ETH-USDC", token='USDC')
print(deleteOrdersRes)

historyOrdersRes = client.history_orders_v2(token='USDC')
print(historyOrdersRes)

getOrderRes = client.get_order_v2(id="123456")
print(getOrderRes)

getOrderRes = client.get_order_by_client_order_id_v2(id="123456")
print(getOrderRes)

fundingRes = client.funding_v2(limit=100,page=0,symbol="BTC-USDC",side="BUY",token='USDC')
print(fundingRes)

notifyListRes = client.notify_list(limit=100,page=0,unreadOnly="true",notifyCategory="1")
print(notifyListRes)

markNotifyReadRes = client.mark_notify_read(ids="113123,123123123")
print(markNotifyReadRes)

historicalPnlRes = client.historical_pnl_v2(limit=100,page=0,beginTimeInclusive=1651406864000,endTimeExclusive=1657105971171,symbol="BTC-USDC")
print(historicalPnlRes)

yesterdayPnlRes = client.yesterday_pnl_v2(token='USDC')
print(yesterdayPnlRes)

historyValueRes = client.history_value_v2(token='USDC')
print(historyValueRes)

markAllNotifyReadRes = client.mark_all_notify_read()
print(markAllNotifyReadRes)

setInitialMarginRateRes = client.set_initial_margin_rate_v2(symbol="BTC-USDC",initialMarginRate="0.1",token='USDC')
print(setInitialMarginRateRes)


print("end, Apexpro")






# ------------ API URLs ------------
APEX_HTTP_MAIN = 'https://pro.apex.exchange'
APEX_HTTP_TEST = 'https://testnet.pro.apex.exchange'
APEX_WS_MAIN = 'wss://quote.pro.apex.exchange'
APEX_WS_TEST = 'wss://quote-qa.pro.apex.exchange'

APEX_OMNI_HTTP_MAIN = 'https://omni.apex.exchange'
APEX_OMNI_HTTP_TEST = 'https://qa.omni.apex.exchange'

APEX_OMNI_WS_MAIN = 'wss://quote.omni.apex.exchange'
APEX_OMNI_WS_TEST = 'wss://qa-quote.omni.apex.exchange'

URL_SUFFIX = "/api"

# ------------ register_env_id ------------
REGISTER_ENVID_MAIN = 1
REGISTER_ENVID_TEST = 5

# ------------ network_id ------------
NETWORKID_MAIN = 1
NETWORKID_TEST = 11155111

NETWORKID_OMNI_MAIN_ARB = 9
NETWORKID_OMNI_TEST_BNB = 3
NETWORKID_OMNI_TEST_BASE = 11

# ------------ Signature Types ------------
SIGNATURE_TYPE_NO_PREPEND = 0
SIGNATURE_TYPE_DECIMAL = 1
SIGNATURE_TYPE_HEXADECIMAL = 2
SIGNATURE_TYPE_PERSONAL = 3


# ------------ Order Side ------------
ORDER_SIDE_BUY = 'BUY'
ORDER_SIDE_SELL = 'SELL'


# ------------ Assets ------------
ASSET_USDC = 'USDC'
ASSET_BTC = 'BTC'
ASSET_ETH = 'ETH'
ASSET_LINK = 'LINK'
ASSET_AAVE = 'AAVE'
ASSET_DOGE = 'DOGE'
ASSET_MATIC = 'MATIC'
COLLATERAL_ASSET = ASSET_USDC

ASSET_RESOLUTION = {
    ASSET_USDC: '1e6',
}

# ------------ Ethereum Transactions ------------
DEFAULT_GAS_AMOUNT = 250000
DEFAULT_GAS_MULTIPLIER = 1.5
DEFAULT_GAS_PRICE = 4000000000
DEFAULT_GAS_PRICE_ADDITION = 3
MAX_SOLIDITY_UINT = 115792089237316195423570985008687907853269984665640564039457584007913129639935  # noqa: E501


COLLATERAL_TOKEN_DECIMALS = 6

# ------------ Off-Chain Ethereum-Signed Actions ------------
OFF_CHAIN_ONBOARDING_ACTION = 'ApeX Onboarding' # action:ApeX Onboarding  onlySignOn:https://pro.apex.exchange nonce:1188491033265307648
OFF_CHAIN_KEY_DERIVATION_ACTION = 'L2 Key' #{"name": "ApeX","version": "1.0","envId": 1,"action": "L2 Key","onlySignOn": "https://pro.apex.exchange"}





from apexpro import HTTP
from apexpro.constants import URL_SUFFIX


class HttpPublic(HTTP):
    def server_time(self, **kwargs):
        """"
        GET Retrieve System Time.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#publicapi-get-apex-server-time
        :returns: Request results as dictionary.
        """

        suffix = URL_SUFFIX + "/v1/time"
        return self._submit_request(
            method="GET",
            path=self.endpoint + suffix
        )

    def depth(self, **kwargs):
        """"
        GET Retrieve Market Depth.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#publicapi-get-retrieve-market-depth
        :returns: Request results as dictionary.
        """
        suffix = URL_SUFFIX + "/v1/depth"
        if kwargs.get('symbol') is not None:
            kwargs['symbol'] = kwargs['symbol'].replace('-', '')
        return self._submit_request(
            method='GET',
            path=self.endpoint + suffix,
            query=kwargs
        )

    def trades(self, **kwargs):
        """"
        GET Retrieve Newest Trading Data.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#publicapi-get-retrieve-newest-trading-data
        :returns: Request results as dictionary.
        """
        suffix = URL_SUFFIX + "/v1/trades"
        if kwargs.get('symbol') is not None:
            kwargs['symbol'] = kwargs['symbol'].replace('-', '')
        return self._submit_request(
            method='GET',
            path=self.endpoint + suffix,
            query=kwargs
        )

    def klines(self, **kwargs):
        """"
        GET Retrieve Candlestick Chart Data.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#publicapi-get-retrieve-candlestick-chart-data
        :returns: Request results as dictionary.
        """
        suffix = URL_SUFFIX + "/v1/klines"
        if kwargs.get('symbol') is not None:
            kwargs['symbol'] = kwargs['symbol'].replace('-', '')
        return self._submit_request(
            method='GET',
            path=self.endpoint + suffix,
            query=kwargs
        )

    def ticker(self, **kwargs):
        """"
        GET Retrieve Ticker Data.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#publicapi-get-retrieve-ticker-data
        :returns: Request results as dictionary.
        """
        suffix = URL_SUFFIX + "/v1/ticker"
        if kwargs.get('symbol') is not None:
            kwargs['symbol'] = kwargs['symbol'].replace('-', '')
        return self._submit_request(
            method='GET',
            path=self.endpoint + suffix,
            query=kwargs
        )

    def history_funding(self, **kwargs):
        """"
        GET Retrieve Funding Rate History.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#publicapi-get-retrieve-funding-rate-history
        :returns: Request results as dictionary.
        """
        suffix = URL_SUFFIX + "/v1/history-funding"
        #if kwargs['symbol'] is not None:
        #    kwargs['symbol'] = kwargs['symbol'].replace('-', '')
        return self._submit_request(
            method='GET',
            path=self.endpoint + suffix,
            query=kwargs
        )

    def history_funding_v2(self, **kwargs):
        """"
        GET Retrieve Funding Rate History.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#publicapi-get-retrieve-funding-rate-history
        :returns: Request results as dictionary.
        """
        suffix = URL_SUFFIX + "/v2/history-funding"
        #if kwargs.get('symbol') is not None:
        #    kwargs['symbol'] = kwargs['symbol'].replace('-', '')
        return self._submit_request(
            method='GET',
            path=self.endpoint + suffix,
            query=kwargs
        )

    def test_ticker(self, **kwargs):
        suffix = URL_SUFFIX + "/v1/test-ticker"
        if kwargs.get('symbol') is not None:
            kwargs['symbol'] = kwargs['symbol'].replace('-', '')
        return self._submit_request(
            method='GET',
            path=self.endpoint + suffix,
            query=kwargs
        )

    def depth_v3(self, **kwargs):
        """"
        GET Retrieve Market Depth.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#publicapi-get-retrieve-market-depth
        :returns: Request results as dictionary.
        """
        suffix = URL_SUFFIX + "/v3/depth"
        if kwargs.get('symbol') is not None:
            kwargs['symbol'] = kwargs['symbol'].replace('-', '')
        return self._submit_request(
            method='GET',
            path=self.endpoint + suffix,
            query=kwargs
        )

    def trades_v3(self, **kwargs):
        """"
        GET Retrieve Newest Trading Data.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#publicapi-get-retrieve-newest-trading-data
        :returns: Request results as dictionary.
        """
        suffix = URL_SUFFIX + "/v3/trades"
        if kwargs.get('symbol') is not None:
            kwargs['symbol'] = kwargs['symbol'].replace('-', '')
        return self._submit_request(
            method='GET',
            path=self.endpoint + suffix,
            query=kwargs
        )

    def klines_v3(self, **kwargs):
        """"
        GET Retrieve Candlestick Chart Data.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#publicapi-get-retrieve-candlestick-chart-data
        :returns: Request results as dictionary.
        """
        suffix = URL_SUFFIX + "/v3/klines"
        if kwargs.get('symbol') is not None:
            kwargs['symbol'] = kwargs['symbol'].replace('-', '')
        return self._submit_request(
            method='GET',
            path=self.endpoint + suffix,
            query=kwargs
        )

    def ticker_v3(self, **kwargs):
        """"
        GET Retrieve Ticker Data.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#publicapi-get-retrieve-ticker-data
        :returns: Request results as dictionary.
        """
        suffix = URL_SUFFIX + "/v3/ticker"
        if kwargs.get('symbol') is not None:
            kwargs['symbol'] = kwargs['symbol'].replace('-', '')
        return self._submit_request(
            method='GET',
            path=self.endpoint + suffix,
            query=kwargs
        )

    def history_funding_v3(self, **kwargs):
        """"
        GET Retrieve Funding Rate History.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#publicapi-get-retrieve-funding-rate-history
        :returns: Request results as dictionary.
        """
        suffix = URL_SUFFIX + "/v3/history-funding"
        return self._submit_request(
            method='GET',
            path=self.endpoint + suffix,
            query=kwargs
        )



import os
import sys

from apexpro.http_private_sign import HttpPrivateSign

root_path = os.path.abspath(__file__)
root_path = '/'.join(root_path.split('/')[:-2])
sys.path.append(root_path)

from apexpro.constants import NETWORKID_TEST, APEX_OMNI_HTTP_TEST

print("Hello, Apex Omni")
# need api_key_credentials={'key': key,'secret': secret, 'passphrase': passphrase} for private api

key = 'your apiKey-key from register V3'
secret = 'your apiKey-secret from register  V3'
passphrase = 'your apiKey-passphrase from register  V3'

seeds = 'your zk seeds from register'
l2Key = 'your l2Key seeds from register'

client = HttpPrivateSign(APEX_OMNI_HTTP_TEST, network_id=NETWORKID_TEST,
                         zk_seeds=seeds,zk_l2Key=l2Key,
                         api_key_credentials={'key': key, 'secret': secret, 'passphrase': passphrase})

configs = client.configs_v3()

userRes = client.get_user_v3()
print(userRes)

accountRes = client.get_account_v3()
print(accountRes)

allApikeys = client.all_apikeys_v3()
print(allApikeys)

nonceRes = client.generate_nonce_v3(refresh="true", l2Key=l2Key,ethAddress=client.default_address, chainId=NETWORKID_TEST)


generateRes = client.generate_api_key_v3(remark="test3",ips="127.0.0.1,127.0.0.2",nonce=nonceRes['data']['nonce'])
print(generateRes)

allApikeys = client.all_apikeys_v3()
print(allApikeys)

print("end, Apex Omni")






import os
import sys
import time

from apexpro.http_private_stark_key_sign import HttpPrivateStark

root_path = os.path.abspath(__file__)
root_path = '/'.join(root_path.split('/')[:-2])
sys.path.append(root_path)

from apexpro.constants import APEX_HTTP_TEST, NETWORKID_TEST, APEX_HTTP_MAIN, NETWORKID_MAIN

print("Hello, Apexpro")

# need api_key_credentials={'key': key,'secret': secret, 'passphrase': passphrase} for private api
# need starkey for withdraw and createOrder


key = 'your apiKey-key from register'
secret = 'your apiKey-secret from register'
passphrase = 'your apiKey-passphrase from register'

public_key = 'your stark_public_key from register'
public_key_y_coordinate = 'your stark_public_key_y_coordinate from register'
private_key = 'your stark_private_key from register'


client = HttpPrivateStark(APEX_HTTP_TEST, network_id=NETWORKID_TEST,
                          stark_public_key=public_key,
                          stark_private_key=private_key,
                          stark_public_key_y_coordinate=public_key_y_coordinate,
                          api_key_credentials={'key': key, 'secret': secret, 'passphrase': passphrase})
configs = client.configs_v2()
client.get_user()
print(client.get_account_v2())

print(client.get_account_balance_v2())

currentTime = time.time()

feeRes = client.uncommon_withdraw_fee_v2(amount='2',chainId='1', token='USDC',)
print(feeRes)
fastWithdrawRes = client.fast_withdrawal_v2(amount='2',expirationEpochSeconds= currentTime,asset='USDC',fee=feeRes['data']['fee'])

createWithdrawRes = client.create_withdrawal_v2(amount='1',expirationEpochSeconds= currentTime,asset='USDC')
print(createWithdrawRes)

historyOrdersRes = client.history_orders_v2(token="USDC")
print(historyOrdersRes)

orderFills = client.order_fills_v2(orderId='498441108374684453')
print(orderFills)

limitFeeRate = '0.0005'

deleteOrdersRes = client.delete_open_orders_v2(token="USDC")
print(deleteOrdersRes)
createOrderRes = client.create_order_v2(symbol="BTC-USDC", side="SELL",
                                           type="LIMIT", size="0.01",expirationEpochSeconds= currentTime,
                                           price="36890", limitFeeRate=limitFeeRate)


#print(createOrderRes)

fillsRes = client.fills_v2(limit=100,page=0,symbol="BTC-USDC",token="USDC")
print(fillsRes)


openOrderRes = client.open_orders_v2(token='USDC')
print(openOrderRes)


deleteOrdersRes = client.delete_open_orders_v2(token="USDC")
print(deleteOrdersRes)

historyOrdersRes = client.history_orders_v2(token="USDC")
print(historyOrdersRes)

openOrderRes = client.open_orders_v2(token='USDC')
print(openOrderRes)

feeRes = client.uncommon_withdraw_fee_v2(amount='2',chainId='97', token='USDC')
print(feeRes)
crossWithdrawRes = client.cross_chain_withdraw_v2(amount='2',expirationEpochSeconds= currentTime,asset='USDC',fee=feeRes['data']['fee'],chainId='97')
print(crossWithdrawRes)

print("end, Apexpro")



from time import sleep

from apexpro.constants import APEX_WS_TEST
from apexpro.websocket_api import WebSocket

key = 'your apiKey-key from register'
secret = 'your apiKey-secret from register'
passphrase = 'your apiKey-passphrase from register'

# Connect with authentication!
ws_client = WebSocket(
    endpoint=APEX_WS_TEST,
    api_key_credentials={'key': key, 'secret': secret, 'passphrase': passphrase},
)

def handle_account(message):
    print(message)
    contents_data = message["contents"]
    print(len(contents_data))

def h1(message):
    print(1, message)
def h2(message):
    print(2, message)
def h3(message):
    print(3, message)
def h4(message):
    print(4, message)

#ws_client.depth_stream(h1,'BTCUSDC',25)
#ws_client.ticker_stream(h2,'BTCUSDC')
#ws_client.trade_stream(h3,'BTCUSDC')
#ws_client.klines_stream(h4,'BTCUSDC',1)
ws_client.account_info_stream(handle_account)


while True:
    # Run your main trading logic here.
    sleep(1)



import decimal
import os
import sys
import time

from apexpro.helpers.util import round_size
from apexpro.http_private_stark_key_sign import HttpPrivateStark

root_path = os.path.abspath(__file__)
root_path = '/'.join(root_path.split('/')[:-2])
sys.path.append(root_path)

from apexpro.constants import APEX_HTTP_TEST, NETWORKID_TEST, APEX_HTTP_MAIN, NETWORKID_MAIN

print("Hello, Apexpro")


# need api_key_credentials={'key': key,'secret': secret, 'passphrase': passphrase} for private api
# need starkey for withdraw and createOrder

key = 'your apiKey-key from register'
secret = 'your apiKey-secret from register'
passphrase = 'your apiKey-passphrase from register'

public_key = 'your stark_public_key from register'
public_key_y_coordinate = 'your stark_public_key_y_coordinate from register'
private_key = 'your stark_private_key from register'


client = HttpPrivateStark(APEX_HTTP_TEST, network_id=NETWORKID_TEST,
                          stark_public_key=public_key,
                          stark_private_key=private_key,
                          stark_public_key_y_coordinate=public_key_y_coordinate,
                          api_key_credentials={'key': key, 'secret': secret, 'passphrase': passphrase})
configs = client.configs()
client.get_user()
print(client.get_account())

# sample1
# When create an order, optimize the size of the order according to the stepSize of the currency symbol,
# and optimize the price of the order according to the tickSize
symbolData = {}
for k, v in enumerate(configs.get('data').get('perpetualContract')):
    if v.get('symbol') == "BTC-USDC":
        symbolData = v

print(round_size("0.0116", symbolData.get('stepSize')))
print(round_size("25555.8", symbolData.get('tickSize')))

# sample2
# Create a limit order
currentTime = time.time()
limitFeeRate = client.account['takerFeeRate']

size = round_size("0.01", symbolData.get('stepSize'))
price = round_size("28888.5", symbolData.get('tickSize'))
createOrderRes = client.create_order(symbol="BTC-USDC", side="BUY",
                                           type="LIMIT", size=size, expirationEpochSeconds= currentTime,
                                           price=price, limitFeeRate=limitFeeRate)
print(createOrderRes)

# sample3
# Create a conditional order

createOrderRes = client.create_order(symbol="ETH-USDC", side="BUY",
                                     type="STOP_LIMIT", size="0.01",expirationEpochSeconds= currentTime,
                                     price="1811.5", limitFeeRate=limitFeeRate, triggerPriceType="INDEX", triggerPrice="1811")
print(createOrderRes)

# sample4
# Create a market order
# first, get a worstPrice from server.( market order price must not none)

worstPrice = client.get_worst_price(symbol="BTC-USDC", side="SELL", size="0.1")
price = worstPrice['data']['worstPrice']

createOrderRes = client.create_order(symbol="BTC-USDC", side="SELL",
                                     type="MARKET", size="1", price=price, limitFeeRate=limitFeeRate,
                                     expirationEpochSeconds= currentTime )
print(createOrderRes)

# sample5
# Create a Position TP/SL order
# first, Set a slippage to get an acceptable price
# if timeInForce="GOOD_TIL_CANCEL" or "POST_ONLY", slippage is recommended to be greater than 0.1
# if timeInForce="FILL_OR_KILL" or "IMMEDIATE_OR_CANCEL", slippage is recommended to be greater than 0.2
# when buying, the price = price*(1 + slippage). when selling, the price = price*(1 - slippage)

slippage = decimal.Decimal("0.1")
price =  round_size(decimal.Decimal("28888") * (decimal.Decimal("1") + slippage), symbolData.get('tickSize'))

createOrderRes = client.create_order(symbol="BTC-USDC", side="BUY", isPositionTpsl = True, reduceOnly= True,
                                     type="TAKE_PROFIT_MARKET", size="0.1", price=price, limitFeeRate=limitFeeRate,
                                     expirationEpochSeconds= currentTime, triggerPriceType="INDEX", triggerPrice="28888" )
print(createOrderRes)

# sample6
# Create a  TP/SL order
# first, Set a slippage to get an acceptable slPrice or tpPrice
#slippage is recommended to be greater than 0.1
# when buying, the price = price*(1 + slippage). when selling, the price = price*(1 - slippage)
slippage = decimal.Decimal("-0.1")
slPrice =  round_size(decimal.Decimal("38000") * (decimal.Decimal("1") + slippage), symbolData.get('tickSize'))
tpPrice =  round_size(decimal.Decimal("49000") * (decimal.Decimal("1") + slippage), symbolData.get('tickSize'))

createOrderRes = client.create_order(symbol="BTC-USDC", side="BUY",
                                     type="LIMIT", size="0.01", expirationEpochSeconds= currentTime,
                                     price="41000", limitFeeRate=limitFeeRate,
                                     isOpenTpslOrder=True,
                                     isSetOpenSl=True,
                                     slPrice=slPrice,
                                     slSide="SELL",
                                     slSize="0.01",
                                     slTriggerPrice="38000",
                                     isSetOpenTp=True,
                                     tpPrice=tpPrice,
                                     tpSide="SELL",
                                     tpSize="0.01",
                                     tpTriggerPrice="49000",
                                     )
print(createOrderRes)


#createWithdrawRes = client.create_withdrawal(amount='1001',expirationEpochSeconds= currentTime,asset='USDC')
#print(createWithdrawRes)

#feeRes = client.uncommon_withdraw_fee(amount='1002',chainId='5')
#print(feeRes)
#fastWithdrawRes = client.fast_withdrawal(amount='1002',expirationEpochSeconds= currentTime,asset='USDC',fee=feeRes['data']['fee'])
#print(fastWithdrawRes)

deleteOrderRes = client.delete_open_orders(symbol="BTC-USDC")
print(deleteOrderRes)

deleteOrderRes = client.delete_open_orders()
print(deleteOrderRes)


feeRes = client.uncommon_withdraw_fee(amount='1003',chainId='97')
print(feeRes)
crossWithdrawRes = client.cross_chain_withdraw(amount='1003',expirationEpochSeconds= currentTime,asset='USDC',fee=feeRes['data']['fee'],chainId='97')
print(crossWithdrawRes)

print("end, Apexpro")





import os
import sys
import time

from apexpro.http_private_v3 import HttpPrivate_v3

root_path = os.path.abspath(__file__)
root_path = '/'.join(root_path.split('/')[:-2])
sys.path.append(root_path)

from apexpro.constants import APEX_OMNI_HTTP_MAIN, NETWORKID_OMNI_MAIN_ARB, NETWORKID_MAIN

print("Hello, Apex Omni")
priKey = "your eth private key"

client = HttpPrivate_v3(APEX_OMNI_HTTP_MAIN, network_id=NETWORKID_MAIN, eth_private_key=priKey)
configs = client.configs_v3()

zkKeys = client.derive_zk_key(client.default_address)
print(zkKeys)
print(zkKeys['seeds'])
print(zkKeys['l2Key'])
print(zkKeys['pubKeyHash'])

nonceRes = client.generate_nonce_v3(refresh="false", l2Key=zkKeys['l2Key'],ethAddress=client.default_address, chainId=NETWORKID_OMNI_MAIN_ARB)

regRes = client.register_user_v3(nonce=nonceRes['data']['nonce'],l2Key=zkKeys['l2Key'], seeds=zkKeys['seeds'],ethereum_address=client.default_address)

print(regRes['data']['apiKey']['key'])
print(regRes['data']['apiKey']['secret'])
print(regRes['data']['apiKey']['passphrase'])

time.sleep(10)
accountRes = client.get_account_v3()
print(accountRes)

#back zkKeys, apiKey,and accountId for private Api or create-order transfer or withdraw

print(regRes['data']['apiKey'])

changeRes = client.change_pub_key_v3(chainId=NETWORKID_OMNI_MAIN_ARB, seeds=zkKeys.get('seeds'), ethPrivateKey=priKey, zkAccountId = accountRes.get('spotAccount').get('zkAccountId'), subAccountId = accountRes.get('spotAccount').get('defaultSubAccountId'),
                                     newPkHash = zkKeys.get('pubKeyHash'),  nonce= accountRes.get('spotAccount').get('nonce'), l2Key= zkKeys.get('l2Key'))
print(changeRes)

time.sleep(10)
accountRes = client.get_account_v3()
print(accountRes)
print("end, Apexpro")




import decimal
import os
import sys
import time

from apexpro.helpers.util import round_size

from apexpro.http_private_sign import HttpPrivateSign
import os
import sys
import time

from apexpro.http_private_sign import HttpPrivateSign

root_path = os.path.abspath(__file__)
root_path = '/'.join(root_path.split('/')[:-2])
sys.path.append(root_path)

from apexpro.constants import NETWORKID_TEST, APEX_OMNI_HTTP_TEST

print("Hello, Apex omni")

key = 'your apiKey-key from register'
secret = 'your apiKey-secret from register'
passphrase = 'your apiKey-passphrase from register'

seeds = 'your zk seeds from register'
l2Key = 'your l2Key seeds from register'


client = HttpPrivateSign(APEX_OMNI_HTTP_TEST, network_id=NETWORKID_TEST,
                          zk_seeds=seeds,zk_l2Key=l2Key,
                          api_key_credentials={'key': key, 'secret': secret, 'passphrase': passphrase})
configs = client.configs_v3()
accountData = client.get_account_v3()


currentTime = time.time()
createOrderRes = client.create_order_v3(symbol="BTC-USDT", side="SELL",
                                        type="MARKET", size="0.001", timestampSeconds= currentTime,
                                        price="60000")
print(createOrderRes)

# sample6
# Create a  TP/SL order
# first, Set a slippage to get an acceptable slPrice or tpPrice
#slippage is recommended to be greater than 0.1
# when buying, the price = price*(1 + slippage). when selling, the price = price*(1 - slippage)
slippage = decimal.Decimal("-0.1")
slPrice =  decimal.Decimal("58000") * (decimal.Decimal("1") + slippage)
tpPrice =  decimal.Decimal("79000") * (decimal.Decimal("1") - slippage)

createOrderRes = client.create_order_v3(symbol="BTC-USDT", side="BUY",
                                     type="LIMIT", size="0.01",
                                     price="65000",
                                     isOpenTpslOrder=True,
                                     isSetOpenSl=True,
                                     slPrice=slPrice,
                                     slSide="SELL",
                                     slSize="0.01",
                                     slTriggerPrice="58000",
                                     isSetOpenTp=True,
                                     tpPrice=tpPrice,
                                     tpSide="SELL",
                                     tpSize="0.01",
                                     tpTriggerPrice="79000",
                                     )
print(createOrderRes)


print("end, Apex Omni")





from time import sleep

from apexpro.constants import APEX_WS_TEST
from apexpro.websocket_api import WebSocket


# Connect with authentication!
ws_client = WebSocket(
    endpoint=APEX_WS_TEST,
)

def depth_data(message):
    showDepthdata(message)


lastDepthData = {}

def _find_index(source,  key):
    """
    Find the index in source list of the targeted ID.
    """
    for item in source:
        if item[0] == key:
            return source.index(item)
    return -1

def showDepthdata(message):
    global lastDepthData
    topic = ''

    if message.get('topic') is not None:
        topic = message.get('topic')

    print("new:",  message)
    if message.get('type') == 'snapshot':
        lastDepthData = message
        print('lastDepthData: ', lastDepthData)
    elif message.get('type') == 'delta':
        if message.get('data').get('u') != lastDepthData.get('data').get('u') + 1:
            ws_client.unsub_depth_topic_stream(depth_data,topic)
            ws_client.depth_topic_stream(depth_data,topic)
        else:
            lastDepthData['u'] = message.get('u')
            if lastDepthData.get('data') is not None and message.get('data') is not None and message.get('data').get('b') is not None:
                for entry in message.get('data').get('b'):
                    index = _find_index(lastDepthData['data']['b'], entry[0])
                    if index == -1:
                        lastDepthData['data']['b'].append(entry)
                    else:
                        lastDepthData['data']['b'][index] = entry
                    if entry[1] == '0':
                        lastDepthData['data']['b'].pop(index)


                items=lastDepthData['data']['b']
                #items.sort()
                sorted(items,key=lambda x:x[0],reverse=False)
            if lastDepthData.get('data') is not None and message.get('data')  is not None and message.get('data').get('a') is not None:
                for entry in message.get('data').get('a'):
                    index = _find_index(lastDepthData['data']['a'], entry[0])
                    if index == -1:
                        lastDepthData['data']['a'].append(entry)
                    else:
                        lastDepthData['data']['a'][index] = entry
                    if entry[1] == '0':
                        lastDepthData['data']['a'].pop(index)


            items=lastDepthData['data']['a']
            #items.sort()
            sorted(items,key=lambda x:x[0],reverse=False)

            print('lastDepthData: ', lastDepthData)
    else:
        ws_client.depth_topic_stream(depth_data,topic)


ws_client.depth_stream(depth_data,'BTCUSDC',25)


while True:
    # Run your main trading logic here.
    sleep(1)



import os
import sys
import time

from apexpro.helpers.util import wait_for_condition
from apexpro.starkex.helpers import nonce_from_client_id

root_path = os.path.abspath(__file__)
root_path = '/'.join(root_path.split('/')[:-2])
sys.path.append(root_path)

from apexpro.http_private import HttpPrivate
from apexpro.constants import APEX_HTTP_TEST, NETWORKID_TEST, APEX_HTTP_MAIN, NETWORKID_MAIN

print("Hello, Apexpro")
priKey = "your eth private key"

key = 'your apiKey-key from register'
secret = 'your apiKey-secret from register'
passphrase = 'your apiKey-passphrase from register'

public_key = 'your stark_public_key from register'
public_key_y_coordinate = 'your stark_public_key_y_coordinate from register'
private_key = 'your stark_private_key from register'


client = HttpPrivate(APEX_HTTP_MAIN, network_id=NETWORKID_MAIN, eth_private_key=priKey,
                     stark_public_key=public_key,
                     stark_private_key=private_key,
                     stark_public_key_y_coordinate=public_key_y_coordinate,
                     api_key_credentials={'key': key, 'secret': secret, 'passphrase': passphrase})
configs = client.configs()

account = client.get_account()

# If you have not approve usdc on eth, please approve first
# Set allowance on the Starkware perpetual contract, for the deposit.
#approve_tx_hash = client.eth.set_token_max_allowance(
#    client.eth.get_exchange_contract().address,
#)
print('Waiting for allowance...')
# Don't worry if you encounter a timeout request while waiting. Execution on the chain takes a certain time
#client.eth.wait_for_tx(approve_tx_hash)
print('...done.')

# Send an on-chain deposit.
deposit_tx_hash = client.eth.deposit_to_exchange(
    client.account['positionId'],
    0.1,
)
print('Waiting for deposit...')
# Don't worry if you encounter a timeout request while waiting. Execution on the chain takes a certain time

client.eth.wait_for_tx(deposit_tx_hash)
print('...done.')






import os
import sys
import time
from threading import Timer

from apexpro.http_private_v3 import HttpPrivate_v3

root_path = os.path.abspath(__file__)
root_path = '/'.join(root_path.split('/')[:-2])
sys.path.append(root_path)

from apexpro.http_private import HttpPrivate
from apexpro.constants import APEX_HTTP_TEST, NETWORKID_TEST, APEX_HTTP_MAIN, NETWORKID_MAIN, APEX_OMNI_HTTP_TEST, \
    APEX_OMNI_HTTP_MAIN, NETWORKID_OMNI_MAIN_ARB

print("Hello, Apex Omni")

key = 'your apiKey-key from register'
secret = 'your apiKey-secret from register'
passphrase = 'your apiKey-passphrase from register'

seeds = 'your zk seeds from register'
l2Key = 'your l2Key seeds from register'
pubKeyHash = 'your l2Key seeds from pubKeyHash'


client = HttpPrivate_v3(APEX_OMNI_HTTP_MAIN, network_id=NETWORKID_OMNI_MAIN_ARB, api_key_credentials={'key': key,'secret': secret, 'passphrase': passphrase})
configs = client.configs_v3()

accountRes = client.get_account_v3()
print(accountRes)

changeRes = client.change_pub_key_v3(chainId=NETWORKID_OMNI_MAIN_ARB, seeds= seeds, zkAccountId = accountRes.get('spotAccount').get('zkAccountId'), subAccountId = accountRes.get('spotAccount').get('defaultSubAccountId'),
                                     newPkHash=pubKeyHash,  nonce=accountRes.get('spotAccount').get('nonce'), l2Key= l2Key, ethSignatureType='Onchain')
print(changeRes)

time.sleep(10)
accountRes = client.get_account_v3()
print(accountRes)
print("end, Apex Omni")




import json

from ._websocket_stream import _identify_ws_method, _make_public_kwargs, _WebSocketManager, \
    _ApexWebSocketManager, PUBLIC_WSS, PRIVATE_WSS

from concurrent.futures import ThreadPoolExecutor



class WebSocket(_ApexWebSocketManager):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self.ws_public = None
        self.ws_private = None
        self.kwargs = kwargs
        self.public_kwargs = _make_public_kwargs(self.kwargs)

    def _ws_public_subscribe(self, sendStr, topic, callback):
        if not self.ws_public:
            self.ws_public = _ApexWebSocketManager(
                **self.public_kwargs)
            self.ws_public._connect(self.endpoint + PUBLIC_WSS)
        self.ws_public.subscribe(sendStr, topic, callback)

    def _ws_private_subscribe(self,topic, callback):
        if not self.ws_private:
            self.ws_private = _ApexWebSocketManager(
                **self.kwargs)
            self.ws_private._connect(self.endpoint + PRIVATE_WSS)
        self.ws_private.subscribe("", topic, callback)

    def custom_topic_stream(self, topic, callback, wss_url):
        subscribe = _identify_ws_method(
            wss_url,
            {
                PUBLIC_WSS: self._ws_public_subscribe,
                PRIVATE_WSS: self._ws_private_subscribe
            })
        subscribe(topic, callback)

    def depth_stream(self, callback, symbol, limit):
        """
        https://api-docs.pro.apex.exchange/#public-websocket-depth
        """
        arg = "orderBook" + str(limit) + ".H." + symbol
        topic = \
            {
                "op": "subscribe",
                "args": [arg]
            }
        topicStr = json.dumps(topic, sort_keys=True, separators=(",", ":"))
        self._ws_public_subscribe(topicStr, arg, callback)
    def unsub_depth_topic_stream(self, callback, arg):
        """
        https://api-docs.pro.apex.exchange/#public-websocket-depth
        """
        topic = \
            {
                "op": "unsubscribe",
                "args": [arg]
            }
        topicStr = json.dumps(topic, sort_keys=True, separators=(",", ":"))
        self._ws_public_subscribe(topicStr, arg, callback)
    def depth_topic_stream(self, callback, arg):
        """
        https://api-docs.pro.apex.exchange/#public-websocket-depth
        """
        topic = \
            {
                "op": "subscribe",
                "args": [arg]
            }
        topicStr = json.dumps(topic, sort_keys=True, separators=(",", ":"))
        self._ws_public_subscribe(topicStr, arg, callback)
    def ticker_stream(self, callback, symbol):
        """
        https://api-docs.pro.apex.exchange/#public-websocket-ticker
        """
        arg = "instrumentInfo" + ".H." + symbol
        topic = \
            {
                "op": "subscribe",
                "args": [arg]
            }
        topicStr = json.dumps(topic, sort_keys=True, separators=(",", ":"))
        self._ws_public_subscribe(topicStr, arg, callback)

    def all_ticker_stream(self, callback):
        """
        https://api-docs.pro.apex.exchange/#public-websocket-ticker
        """
        arg = "instrumentInfo.all"
        topic = \
            {
                "op": "subscribe",
                "args": [arg]
            }
        topicStr = json.dumps(topic, sort_keys=True, separators=(",", ":"))
        self._ws_public_subscribe(topicStr, arg, callback)

    def klines_stream(self, callback, symbol, interval):
        """
        https://api-docs.pro.apex.exchange/#public-websocket-candlestick-chart
        """
        arg = "candle" + "." + str(interval) + "." + symbol
        topic = \
            {
                "op": "subscribe",
                "args": [arg]
            }
        topicStr = json.dumps(topic, sort_keys=True, separators=(",", ":"))
        self._ws_public_subscribe(topicStr, arg, callback)

    def trade_stream(self, callback, symbol):
        """
       https://api-docs.pro.apex.exchange/#public-websocket-trade
        """
        arg = "recentlyTrade" + ".H." + symbol
        topic = \
            {
                "op": "subscribe",
                "args": [arg]
            }
        topicStr = json.dumps(topic, sort_keys=True, separators=(",", ":"))
        self._ws_public_subscribe(topicStr, arg, callback)


    def account_info_stream(self, callback):
        """
        https://api-docs.pro.apex.exchange/#private-websocket
        """
        topic = "ws_accounts_v1"
        self._ws_private_subscribe(topic=topic, callback=callback)

    def account_info_stream_v2(self, callback):
        """
        https://api-docs.pro.apex.exchange/#private-websocket
        """
        topic = "ws_accounts_v2"
        self._ws_private_subscribe(topic=topic, callback=callback)
    def account_info_stream_v3(self, callback):
        """
        https://api-docs.pro.apex.exchange/#private-websocket
        """
        topic = "ws_zk_accounts_v3"
        self._ws_private_subscribe(topic=topic, callback=callback)



import base64
import hashlib
import hmac

from web3 import Web3

from apexpro import private_key_to_public_key_pair_hex
from apexpro.constants import URL_SUFFIX, OFF_CHAIN_KEY_DERIVATION_ACTION, OFF_CHAIN_ONBOARDING_ACTION
from apexpro.helpers.request_helpers import generate_query_path, \
    generate_now
from apexpro.http_public import HttpPublic


class HttpPrivate(HttpPublic):
    def _private_request(
            self,
            method,
            path,
            data={},
            headers=None
    ):
        now_iso = generate_now()
        if self.api_key_credentials is not None:
            signature = self.sign(
                request_path=path,
                method=method.upper(),
                iso_timestamp=str(now_iso),
                data=data,
            )
            headers = {
                'APEX-SIGNATURE': signature,
                'APEX-API-KEY': self.api_key_credentials.get('key'),
                'APEX-TIMESTAMP': str(now_iso),
                'APEX-PASSPHRASE': self.api_key_credentials.get('passphrase'),
            }
        return self._submit_request(
            method=method,
            path=self.endpoint + path,
            headers=headers,
            query=data,
        )

    def _get(self, endpoint, params):
        return self._private_request(
            'GET',
            generate_query_path(endpoint, params),
        )

    def _post(self, endpoint, data, headers=None):
        return self._private_request(
            'POST',
            endpoint,
            data,
            headers
        )

    # ============ Signing ============

    def sign(
            self,
            request_path,
            method,
            iso_timestamp,
            data,
    ):
        sortedItems=sorted(data.items(),key=lambda x:x[0],reverse=False)
        dataString = '&'.join('{key}={value}'.format(
            key=x[0], value=x[1]) for x in sortedItems if x[1] is not None)

        message_string = (
                iso_timestamp +
                method +
                request_path +
                dataString
        )
        hashed = hmac.new(
            base64.standard_b64encode(
                (self.api_key_credentials.get('secret')).encode(encoding='utf-8'),
            ),
            msg=message_string.encode(encoding='utf-8'),
            digestmod=hashlib.sha256,
        )
        return base64.standard_b64encode(hashed.digest()).decode()

    def generate_nonce(self, starkKey, ethAddress, chainId,
                       refresh="false"
                       ):
        """"
        POST: Generate nonce.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-post-generate-nonce
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/generate-nonce"
        return self._private_request(
            method="POST",
            path=path,
            data={
                'starkKey': starkKey,
                'ethAddress': ethAddress,
                'chainId': chainId,
                'category': 'CATEGORY_API',
                'refresh': refresh,
            }
        )

    def register_user(
            self,
            nonce,
            starkKey=None,
            stark_public_key_y_coordinate=None,
            ethereum_address=None,
            referred_by_affiliate_link=None,
            country=None,
            isLpAccount=None,
            eth_mul_address=None,
            sourceFlag=None,
    ):
        """"
        POST Registration & Onboarding.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-post-registration-amp-onboarding
        :returns: Request results as dictionary.
        """
        stark_key = starkKey or self.stark_public_key
        stark_key_y = (
                stark_public_key_y_coordinate or self.stark_public_key_y_coordinate
        )
        if stark_key is None:
            raise ValueError(
                'STARK private key or public key is required'
            )
        if stark_key_y is None:
            raise ValueError(
                'STARK private key or public key y-coordinate is required'
            )

        eth_address = ethereum_address or self.default_address
        signature = self.signer.sign(
            eth_address,
            action=OFF_CHAIN_ONBOARDING_ACTION,
            nonce=nonce,
        )

        path = URL_SUFFIX + "/v1/onboarding"
        onboardingRes = self._private_request(
            method="POST",
            path=path,
            data= {
                'starkKey': stark_key,
                'starkKeyYCoordinate': stark_key_y,
                'referredByAffiliateLink': referred_by_affiliate_link,
                'ethereumAddress': eth_address,
                'country': country,
                'category': 'CATEGORY_API',
                'isLpAccount': isLpAccount,
                'ethMulAddress': eth_mul_address,
                'sourceFlag': sourceFlag,
            },
            headers={
                'APEX-SIGNATURE': signature,
                'APEX-ETHEREUM-ADDRESS': eth_address,
            }
        )
        if onboardingRes.get('data') is not None:
            self.user = onboardingRes.get('data').get('user')
            self.account = onboardingRes.get('data').get('account')
        return onboardingRes

    def register_user_v2(
            self,
            nonce,
            starkKey=None,
            stark_public_key_y_coordinate=None,
            ethereum_address=None,
            referred_by_affiliate_link=None,
            country=None,
            isLpAccount=None,
            eth_mul_address=None,
            token=None,
            sourceFlag=None,
    ):
        """"
        POST Registration & Onboarding.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-post-registration-amp-onboarding
        :returns: Request results as dictionary.
        """
        stark_key = starkKey or self.stark_public_key
        stark_key_y = (
                stark_public_key_y_coordinate or self.stark_public_key_y_coordinate
        )
        if stark_key is None:
            raise ValueError(
                'STARK private key or public key is required'
            )
        if stark_key_y is None:
            raise ValueError(
                'STARK private key or public key y-coordinate is required'
            )

        eth_address = ethereum_address or self.default_address
        signature = self.signer.sign(
            eth_address,
            action=OFF_CHAIN_ONBOARDING_ACTION,
            nonce=nonce,
        )

        path = URL_SUFFIX + "/v2/onboarding-all"
        onboardingRes = self._private_request(
            method="POST",
            path=path,
            data= {
                'starkKey': stark_key,
                'starkKeyYCoordinate': stark_key_y,
                'referredByAffiliateLink': referred_by_affiliate_link,
                'ethereumAddress': eth_address,
                'country': country,
                'category': 'CATEGORY_API',
                'isLpAccount': isLpAccount,
                'ethMulAddress': eth_mul_address,
                'token':token,
                'sourceFlag': sourceFlag,
            },
            headers={
                'APEX-SIGNATURE': signature,
                'APEX-ETHEREUM-ADDRESS': eth_address,
            }
        )
        if onboardingRes.get('data') is not None:
            self.user = onboardingRes.get('data').get('user')
            self.account = onboardingRes.get('data').get('account')
        return onboardingRes

    def derive_stark_key(
            self,
            ethereum_address=None,
    ):
        signature = self.starkeySigner.sign_message(
            ethereum_address or self.default_address,
            action=OFF_CHAIN_KEY_DERIVATION_ACTION,
            )
        signature_int = int(signature, 16)
        hashed_signature = Web3.solidityKeccak(['uint256'], [signature_int])
        private_key_int = int(hashed_signature.hex(), 16) >> 5
        private_key_hex = hex(private_key_int)
        public_x, public_y = private_key_to_public_key_pair_hex(
            private_key_hex,
        )
        return {
            'public_key': public_x,
            'public_key_y_coordinate': public_y,
            'private_key': private_key_hex
        }


    def recover_api_key_credentials(
            self,
            nonce,
            ethereum_address=None,
    ):
        '''
        Derive API credentials deterministically from an Ethereum key.
        '''
        signature = self.signer.sign(
            ethereum_address or self.default_address,
            action=OFF_CHAIN_ONBOARDING_ACTION,
            nonce=nonce,
        )
        r_hex = signature[2:66]
        r_int = int(r_hex, 16)
        hashed_r_bytes = bytes(Web3.solidityKeccak(['uint256'], [r_int]))
        secret_bytes = hashed_r_bytes[:30]
        s_hex = signature[66:130]
        s_int = int(s_hex, 16)
        hashed_s_bytes = bytes(Web3.solidityKeccak(['uint256'], [s_int]))
        key_bytes = hashed_s_bytes[:16]
        passphrase_bytes = hashed_s_bytes[16:31]

        key_hex = key_bytes.hex()
        key_uuid = '-'.join([
            key_hex[:8],
            key_hex[8:12],
            key_hex[12:16],
            key_hex[16:20],
            key_hex[20:],
        ])

        return {
            'secret': base64.urlsafe_b64encode(secret_bytes).decode(),
            'key': key_uuid,
            'passphrase': base64.urlsafe_b64encode(passphrase_bytes).decode(),
        }

    def user(self, **kwargs):
        """"
        GET Retrieve User Data.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-user-data
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/user"
        userRes = self._get(
            endpoint=path,
            params=kwargs
        )
        self.user = userRes.get('data')
        return userRes

    def get_user(self, **kwargs):
        """"
        GET Retrieve User Data.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-user-data
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/user"
        userRes = self._get(
            endpoint=path,
            params=kwargs
        )
        self.user = userRes.get('data')
        return userRes


    def modify_user(self, **kwargs):
        """"
        POST Edit User Data.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-post-edit-user-data
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/modify-user"

        return self._post(
            endpoint=path,
            data=kwargs
        )

    def account(self, **kwargs):
        """"
        GET Retrieve User Account Data.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-user-account-data
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/account"
        accountRes =  self._get(
            endpoint=path,
            params=kwargs
        )
        self.account = accountRes.get('data')
        return accountRes

    def get_account(self, **kwargs):
        """"
        GET Retrieve User Account Data.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-user-account-data
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/account"
        accountRes =  self._get(
            endpoint=path,
            params=kwargs
        )
        self.account = accountRes.get('data')
        return accountRes

    def get_account_v2(self, **kwargs):
        """"
        GET Retrieve User Account Data.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-user-account-data
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/account"
        accountRes =  self._get(
            endpoint=path,
            params=kwargs
        )
        self.account = accountRes.get('data')
        return accountRes

    def transfers(self, **kwargs):
        """"
        GET Retrieve User Deposit Data.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-user-deposit-data
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/transfers"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def transfers_v2(self, **kwargs):
        """"
        GET Retrieve User Deposit Data.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-user-deposit-data
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/transfers"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def withdraw_list(self, **kwargs):
        """"
        GET Retrieve User Withdrawal List.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-user-withdrawal-list
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/withdraw-list"
        return self._get(
            endpoint=path,
            params=kwargs
        )
    def withdraw_list_v2(self, **kwargs):
        """"
        GET Retrieve User Withdrawal List.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-user-withdrawal-list
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/withdraw-list"
        return self._get(
            endpoint=path,
            params=kwargs
        )
    def uncommon_withdraw_fee(self, **kwargs):
        """"
        GET Fast & Cross-Chain Withdrawal Fees.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-fast-amp-cross-chain-withdrawal-fees
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/uncommon-withdraw-fee"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def uncommon_withdraw_fee_v2(self, **kwargs):
        """"
        GET Fast & Cross-Chain Withdrawal Fees.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-fast-amp-cross-chain-withdrawal-fees
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/uncommon-withdraw-fee"
        return self._get(
            endpoint=path,
            params=kwargs
        )


    def transfer_limit(self, **kwargs):
        """"
        GET Retrieve Withdrawal & Transfer Limits.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-withdrawal-amp-transfer-limits
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/transfer-limit"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def transfer_limit_v2(self, **kwargs):
        """"
        GET Retrieve Withdrawal & Transfer Limits.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-withdrawal-amp-transfer-limits
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/transfer-limit"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def fills(self, **kwargs):
        """"
        GET Retrieve Trade History.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-trade-history
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/fills"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def fills_v2(self, **kwargs):
        """"
        GET Retrieve Trade History.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-trade-history
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/fills"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def order_fills_v2(self, **kwargs):

        path = URL_SUFFIX + "/v2/order-fills"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def delete_order(self, **kwargs):
        """"
        POST Cancel Order.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-post-cancel-order
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/delete-order"
        return self._post(
            endpoint=path,
            data=kwargs
        )


    def delete_order_v2(self, **kwargs):
        """"
        POST Cancel Order.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-post-cancel-order
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/delete-order"
        return self._post(
            endpoint=path,
            data=kwargs
        )

    def delete_order_by_client_order_id(self, **kwargs):
        """"
        POST Cancel Order.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-post-cancel-order
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/delete-client-order-id"
        return self._post(
            endpoint=path,
            data=kwargs
        )
    def delete_order_by_client_order_id_v2(self, **kwargs):
        """"
        POST Cancel Order.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-post-cancel-order
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/delete-client-order-id"
        return self._post(
            endpoint=path,
            data=kwargs
        )

    def delete_open_orders(self, **kwargs):
        """"
        POST Cancel all Open Orders
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-post-cancel-all-open-orders
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/delete-open-orders"
        return self._post(
            endpoint=path,
            data=kwargs
        )

    def delete_open_orders_v2(self, **kwargs):
        """"
        POST Cancel all Open Orders
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-post-cancel-all-open-orders
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/delete-open-orders"
        return self._post(
            endpoint=path,
            data=kwargs
        )
    def open_orders(self, **kwargs):
        """"
        GET Retrieve Open Orders.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-open-orders
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/open-orders"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def open_orders_v2(self, **kwargs):
        """"
        GET Retrieve Open Orders.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-open-orders
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/open-orders"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def history_orders(self, **kwargs):
        """"
        GET Retrieve All Order History.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-all-order-history
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/history-orders"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def history_orders_v2(self, **kwargs):
        """"
        GET Retrieve All Order History.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-all-order-history
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/history-orders"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def get_order(self, **kwargs):
        """"
        GET Retrieve Order ID.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-order-id
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/get-order"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def get_order_v2(self, **kwargs):
        """"
        GET Retrieve Order ID.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-order-id
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/get-order"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def get_order_by_client_order_id(self, **kwargs):
        """"
        GET Retrieve Order ID.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-order-id
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/order-by-client-id"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def get_order_by_client_order_id_v2(self, **kwargs):
        """"
        GET Retrieve Order ID.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-order-id
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/order-by-client-id"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def funding(self, **kwargs):
        """"
        GET Retrieve Funding Rate.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-funding-rate
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/funding"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def funding_v2(self, **kwargs):
        """"
        GET Retrieve Funding Rate.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-funding-rate
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/funding"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def notify_list(self, **kwargs):
        """"
        GET Retrieve Notification List.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-notification-list
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/notify-list"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def mark_notify_read(self, **kwargs):
        """"
        POST Mark Notification As Read.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-post-mark-notification-as-read
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/mark-notify-read"
        return self._post(
            endpoint=path,
            data=kwargs
        )

    def historical_pnl(self, **kwargs):
        """"
        GET Retrieve User Historial Profit and Loss.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-user-historial-profit-and-loss
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/historical-pnl"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def historical_pnl_v2(self, **kwargs):
        """"
        GET Retrieve User Historial Profit and Loss.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-user-historial-profit-and-loss
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/historical-pnl"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def yesterday_pnl(self, **kwargs):
        """"
        GET Retrieve Yesterday's Profit & Loss.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-yesterday-39-s-profit-amp-loss
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/yesterday-pnl"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def yesterday_pnl_v2(self, **kwargs):
        """"
        GET Retrieve Yesterday's Profit & Loss.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-yesterday-39-s-profit-amp-loss
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/yesterday-pnl"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def history_value(self, **kwargs):
        """"
        GET Retrieve Historical Asset Value.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-historical-asset-value
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/history-value"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def history_value_v2(self, **kwargs):
        """"
        GET Retrieve Historical Asset Value.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-historical-asset-value
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/history-value"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def mark_all_notify_read(self, **kwargs):
        """"
        POST Mark All Notifications As Read.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-post-mark-all-notifications-as-read
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/mark-all-notify-read"
        return self._post(
            endpoint=path,
            data=kwargs
        )

    def mark_all_notify_read(self, **kwargs):
        """"
        POST Mark All Notifications As Read.
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-post-mark-all-notifications-as-read
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/mark-all-notify-read"
        return self._post(
            endpoint=path,
            data=kwargs
        )

    def get_worst_price(self, **kwargs):
        """"
        get market price from orderbook
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-worst-price
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/get-worst-price"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def get_order_by_client_id(self, **kwargs):
        """"
        get market price from orderbook
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-worst-price
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/order-by-client-id"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def get_account_balance(self, **kwargs):
        """"
        get market price from orderbook
        :param kwargs: See
        https://api-docs.pro.apex.exchange/?lang=zh-TW#privateapi-get-retrieve-user-account-balance
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/account-balance"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def get_account_balance_v2(self, **kwargs):
        """"
        get market price from orderbook
        :param kwargs: See
        https://api-docs.pro.apex.exchange/?lang=zh-TW#privateapi-get-retrieve-user-account-balance
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/account-balance"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def gray_status(self, **kwargs):
        """"
        get market price from orderbook
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-get-retrieve-worst-price
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/gray-status"
        return self._get(
            endpoint=path,
            params=kwargs
        )

    def set_initial_margin_rate(self, **kwargs):
        """"
        get market price from orderbook
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-post-sets-the-initial-margin-rate-of-a-contract
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v1/set-initial-margin-rate"
        return self._post(
            endpoint=path,
            data=kwargs
        )

    def set_initial_margin_rate_v2(self, **kwargs):
        """"
        get market price from orderbook
        :param kwargs: See
        https://api-docs.pro.apex.exchange/#privateapi-post-sets-the-initial-margin-rate-of-a-contract
        :returns: Request results as dictionary.
        """

        path = URL_SUFFIX + "/v2/set-initial-margin-rate"
        return self._post(
            endpoint=path,
            data=kwargs
        )

    def all_favorites(self, **kwargs):

        path = URL_SUFFIX + "/v1/all-favorites"
        return self._get(
            endpoint=path,
            params=kwargs
        )




import os
import sys
import urllib.parse

from apexpro.helpers.request_helpers import random_client_id

from apexpro.http_private_sign import HttpPrivateSign
import os
import sys

from apexpro.http_private_sign import HttpPrivateSign

root_path = os.path.abspath(__file__)
root_path = '/'.join(root_path.split('/')[:-2])
sys.path.append(root_path)

from apexpro.constants import NETWORKID_TEST, APEX_OMNI_HTTP_TEST

print("Hello, Apex Omni")

key = 'your apiKey-key from register'
secret = 'your apiKey-secret from register'
passphrase = 'your apiKey-passphrase from register'

seeds = 'your zk seeds from register'
l2Key = 'your l2Key seeds from register'


client = HttpPrivateSign(APEX_OMNI_HTTP_TEST, network_id=NETWORKID_TEST,
                          zk_seeds=seeds,zk_l2Key=l2Key,
                          api_key_credentials={'key': key, 'secret': secret, 'passphrase': passphrase})
configs = client.configs_v3()
accountData = client.get_account_v3()

#smple1 withdraw
#createWithdrawRes = client.create_withdrawal_v3(amount='3',asset='USDT', toChainId=3)
#print(createWithdrawRes)

#smple2 fast withdraw
#withdraw_feeRes = client.withdraw_fee_v3(amount="3",chainIds="3",tokenId='140')
#print(withdraw_feeRes)
#createWithdrawRes = client.create_withdrawal_v3(amount='3',asset='USDT', toChainId=3, fee=withdraw_feeRes.get('data').get('withdrawFeeAndPoolBalances')[0].get('fee'), isFastWithdraw=True)
#print(createWithdrawRes)

#smple3 transfer_out,  from fund account to contract account
#createTransferRes = client.create_transfer_out_v3(amount='3.4359738368',asset='USDT')
#print(createTransferRes)

#createTransferRes = client.create_transfer_out_v3(amount='0.01',asset='ETH')
#print(createTransferRes)

#smple4 contract transfer_out, from contract account to fund account
#createContractTransferRes = client.create_contract_transfer_out_v3(amount='0.005',asset='ETH')
#print(createContractTransferRes)

#smple5 contract contract_transfer_to_address,   from one contract account to another contract account
#createContractTransferRes = client.create_contract_transfer_to_address_v3(amount='1.1',asset='USDT',receiverAddress='0xfab6256aeef3be7805d3138be8fe1369f716ebc5',receiverAccountId='585750146675900485',receiverL2Key='0x04a234f299958150707451f649208fd085680bf3e1be432acb533eb2cc06082a')
#print(createContractTransferRes)

#smple6 manual-create-repayment

#clientId = random_client_id()
#repaymentPriceRes = client.get_repayment_price_v3(repaymentPriceTokens='ETH|0.001', clientId=clientId)
#print(repaymentPriceRes)
# the clientId for create_manual_repayment_v3 is the same as get_repayment_price_v3.
#repaymentTokens = repaymentPriceRes.get('data').get('repaymentTokens')[0].get('token')+'|'+repaymentPriceRes.get('data').get('repaymentTokens')[0].get('price')+'|'+repaymentPriceRes.get('data').get('repaymentTokens')[0].get('size')
#repaymentRes = client.create_manual_repayment_v3(repaymentTokens=repaymentTokens, poolRepaymentTokens=repaymentTokens, clientId=clientId)
#print(repaymentRes)

print("end, Apexpro")





import os
import sys

root_path = os.path.abspath(__file__)
root_path = '/'.join(root_path.split('/')[:-2])
sys.path.append(root_path)


from apexpro.constants import APEX_HTTP_TEST, APEX_HTTP_MAIN
from apexpro.http_public import HttpPublic

print("Hello, Apexpro")

client = HttpPublic(APEX_HTTP_MAIN)
print(client.history_funding_v2(symbol="BTC-USDT"))
print(client.klines(symbol="ETHUSDT",interval=5,start=1681463600, end=1681563600, limit=5))
print(client.server_time())
print(client.configs_v2())
print(client.depth(symbol="BTC-USDC"))
print(client.trades(symbol="BTC-USDC"))
print(client.klines(symbol="BTC-USDT",interval="15"))
print(client.ticker(symbol="BTC-USDT"))
print(client.history_funding(symbol="BTC-USDT"))

print(client.depth(symbol="ETH-USDT",limit=50))
print(client.trades(symbol="ETH-USDT",limit=50))
print(client.klines(symbol="ETH-USDT",interval="15"))
print(client.history_funding_v2(symbol="ETH-USDT",limit=100,page=0,beginTimeInclusive=1662348573000,endTimeExclusive=1662434973000))




import os
import sys
from datetime import time

from apexpro.http_private_v3 import HttpPrivate_v3

root_path = os.path.abspath(__file__)
root_path = '/'.join(root_path.split('/')[:-2])
sys.path.append(root_path)

from apexpro.constants import APEX_OMNI_HTTP_MAIN, \
    NETWORKID_OMNI_MAIN_ARB

print("Hello, Apex Omni")
# need api_key_credentials={'key': key,'secret': secret, 'passphrase': passphrase} for private api

key = 'your apiKey-key from register V3'
secret = 'your apiKey-secret from register  V3'
passphrase = 'your apiKey-passphrase from register  V3'

client = HttpPrivate_v3(APEX_OMNI_HTTP_MAIN, network_id=NETWORKID_OMNI_MAIN_ARB, api_key_credentials={'key': key,'secret': secret, 'passphrase': passphrase})
configs = client.configs_v3()

userRes = client.get_user_v3()
print(userRes)


accountRes = client.get_account_v3()
print(accountRes)

currentTime = time.time()*1000
withdrawRes = client.withdraws_by_time_and_status_v3(startTime=int(currentTime), status='success')

accountBalanceRes = client.get_account_balance_v3()
print(accountBalanceRes)

fillsRes = client.fills_v3(limit=100,page=0,symbol="BTC-USDT",side="BUY",token="USDT")
print(fillsRes)

transfersRes = client.transfers_v3(limit=100)
print(transfersRes)

transferRes = client.transfer_v3(ids='586213648326721628')
print(transferRes)

transfersRes = client.contract_transfers_v3(limit=100)
print(transfersRes)

transferRes = client.contract_transfer_v3(ids='588301879870489180')
print(transferRes)

#deleteOrderRes = client.delete_order_v3(id="588302655921587036")
#print(deleteOrderRes)

#deleteOrderRes = client.delete_order_by_client_order_id_v3(id="123456")
#print(deleteOrderRes)

openOrdersRes = client.open_orders_v3()
print(openOrdersRes)

deleteOrdersRes = client.delete_open_orders_v3(symbol="BTC-USDT",)
print(deleteOrdersRes)

historyOrdersRes = client.history_orders_v3(token='USDT')
print(historyOrdersRes)

getOrderRes = client.get_order_v3(id="123456")
print(getOrderRes)

getOrderRes = client.get_order_by_client_order_id_v3(id="123456")
print(getOrderRes)

fundingRes = client.funding_v3(limit=100)
print(fundingRes)

historicalPnlRes = client.historical_pnl_v3(limit=100)
print(historicalPnlRes)

yesterdayPnlRes = client.yesterday_pnl_v3()
print(yesterdayPnlRes)

historyValueRes = client.history_value_v3()
print(historyValueRes)

setInitialMarginRateRes = client.set_initial_margin_rate_v3(symbol="BTC-USDT",initialMarginRate="0.05")
print(setInitialMarginRateRes)


print("end, Apex Omni")



